/*
 * Do not modify this file. Changes will be overwritten.
 *
 * Generated automatically from make-plugin-reg.py.
 */

#include "config.h"

#include <gmodule.h>

/* plugins are DLLs on Windows */
#define WS_BUILD_DLL
#include "ws_symbol_export.h"
#include <wsutil/plugins.h>

#include "wsutil/codecs.h"

void codec_register_g729(void);

#include <wsutil/plugin_exports.h>

WS_DLL_PUBLIC_DEF const char plugin_version[] = PLUGIN_VERSION;
WS_DLL_PUBLIC_DEF const int plugin_want_major = VERSION_MAJOR;
WS_DLL_PUBLIC_DEF const int plugin_want_minor = VERSION_MINOR;

uint32_t plugin_describe(void)
{
    return WS_PLUGIN_DESC_CODEC;
}

void plugin_register(void)
{
    static codecs_plugin plug_g729;

    plug_g729.register_codec_module = codec_register_g729;
    codecs_register_plugin(&plug_g729);
}
