/****************************************************************************
** Meta object code from reading C++ file 'plot_dialog.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ui/qt/plot_dialog.h"
#include <QtGui/qtextcursor.h>
#include <QtGui/qscreen.h>
#include <QtCore/qmetatype.h>
#include <QtCore/QList>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'plot_dialog.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN10PlotDialogE_t {};
} // unnamed namespace

template <> constexpr inline auto PlotDialog::qt_create_metaobjectdata<qt_meta_tag_ZN10PlotDialogE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "PlotDialog",
        "goToPacket",
        "",
        "packet",
        "hf_id",
        "updateMarker",
        "size",
        "xCoord",
        "setPosMarker",
        "selectMPos",
        "posMPos",
        "scheduleReplot",
        "scheduleRecalc",
        "scheduleRetap",
        "modelDataChanged",
        "QModelIndex",
        "topLeft",
        "bottomRight",
        "QList<int>",
        "roles",
        "modelRowsReset",
        "modelRowsInserted",
        "parent",
        "first",
        "last",
        "modelRowsRemoved",
        "modelRowsMoved",
        "sourceParent",
        "sourceStart",
        "sourceEnd",
        "destinationParent",
        "destinationRow",
        "onThemeChanged",
        "applyChanges",
        "updateStatistics",
        "updateLegend",
        "copyFromProfile",
        "filename",
        "copyAsCsvClicked",
        "showContextMenu",
        "QPoint",
        "pos",
        "moveLegend",
        "graphClicked",
        "QMouseEvent*",
        "event",
        "mouseMoved",
        "mouseReleased",
        "selectedFrameChanged",
        "frames",
        "plotUatSelectionChanged",
        "QItemSelection",
        "selected",
        "deselected",
        "on_leftButtonBox_clicked",
        "QAbstractButton*",
        "button",
        "on_actionLegend_triggered",
        "checked",
        "on_actionLogScale_triggered",
        "on_actionCrosshairs_triggered",
        "on_actionTopAxis_triggered",
        "on_automaticUpdateCheckBox_toggled",
        "on_plotUat_currentItemChanged",
        "current",
        "previous",
        "on_actionGoToPacket_triggered",
        "on_newToolButton_clicked",
        "on_deleteToolButton_clicked",
        "on_copyToolButton_clicked",
        "on_clearToolButton_clicked",
        "on_moveUpwardsToolButton_clicked",
        "on_moveDownwardsToolButton_clicked",
        "on_rightButtonBox_helpRequested",
        "on_actionReset_triggered",
        "on_actionZoomIn_triggered",
        "on_actionZoomInX_triggered",
        "on_actionZoomInY_triggered",
        "on_actionZoomOut_triggered",
        "on_actionZoomOutX_triggered",
        "on_actionZoomOutY_triggered",
        "on_actionMoveUp10_triggered",
        "on_actionMoveLeft10_triggered",
        "on_actionMoveRight10_triggered",
        "on_actionMoveDown10_triggered",
        "on_actionMoveUp1_triggered",
        "on_actionMoveLeft1_triggered",
        "on_actionMoveRight1_triggered",
        "on_actionMoveDown1_triggered",
        "on_actionMoveUp100_triggered",
        "on_actionMoveLeft100_triggered",
        "on_actionMoveRight100_triggered",
        "on_actionMoveDown100_triggered",
        "on_actionToggleTimeOrigin_triggered",
        "on_rightButtonBox_accepted",
        "on_actionAutoScroll_triggered",
        "on_actionEnableMultiYAxes_triggered",
        "on_actionAddMarker_triggered",
        "on_actionMoveMarker_triggered",
        "on_actionShowPosMarker_triggered",
        "on_actionShowMarkersDifference_triggered",
        "on_actionDeleteMarker_triggered",
        "on_actionDeleteAllMarkers_triggered",
        "on_actionShowDataPointMarker_triggered"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'goToPacket'
        QtMocHelpers::SignalData<void(int, int)>(1, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 3 }, { QMetaType::Int, 4 },
        }}),
        // Signal 'updateMarker'
        QtMocHelpers::SignalData<void(const int, const double, const int)>(5, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 6 }, { QMetaType::Double, 7 }, { QMetaType::Int, 2 },
        }}),
        // Signal 'setPosMarker'
        QtMocHelpers::SignalData<void(const double, const int, const int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Double, 7 }, { QMetaType::Int, 9 }, { QMetaType::Int, 10 },
        }}),
        // Slot 'scheduleReplot'
        QtMocHelpers::SlotData<void()>(11, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'scheduleRecalc'
        QtMocHelpers::SlotData<void()>(12, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'scheduleRetap'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'modelDataChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &, const QVector<int> &)>(14, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 16 }, { 0x80000000 | 15, 17 }, { 0x80000000 | 18, 19 },
        }}),
        // Slot 'modelRowsReset'
        QtMocHelpers::SlotData<void()>(20, 2, QMC::AccessProtected, QMetaType::Void),
        // Slot 'modelRowsInserted'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(21, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'modelRowsRemoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int)>(25, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 22 }, { QMetaType::Int, 23 }, { QMetaType::Int, 24 },
        }}),
        // Slot 'modelRowsMoved'
        QtMocHelpers::SlotData<void(const QModelIndex &, int, int, const QModelIndex &, int)>(26, 2, QMC::AccessProtected, QMetaType::Void, {{
            { 0x80000000 | 15, 27 }, { QMetaType::Int, 28 }, { QMetaType::Int, 29 }, { 0x80000000 | 15, 30 },
            { QMetaType::Int, 31 },
        }}),
        // Slot 'onThemeChanged'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'applyChanges'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateStatistics'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'updateLegend'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'copyFromProfile'
        QtMocHelpers::SlotData<void(const QString &)>(36, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::QString, 37 },
        }}),
        // Slot 'copyAsCsvClicked'
        QtMocHelpers::SlotData<void()>(38, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'showContextMenu'
        QtMocHelpers::SlotData<void(const QPoint &)>(39, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 40, 41 },
        }}),
        // Slot 'moveLegend'
        QtMocHelpers::SlotData<void()>(42, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'graphClicked'
        QtMocHelpers::SlotData<void(QMouseEvent *)>(43, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'mouseMoved'
        QtMocHelpers::SlotData<void(QMouseEvent *)>(46, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'mouseReleased'
        QtMocHelpers::SlotData<void(QMouseEvent *)>(47, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 44, 45 },
        }}),
        // Slot 'selectedFrameChanged'
        QtMocHelpers::SlotData<void(const QList<int> &)>(48, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 18, 49 },
        }}),
        // Slot 'plotUatSelectionChanged'
        QtMocHelpers::SlotData<void(const QItemSelection &, const QItemSelection &)>(50, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 51, 52 }, { 0x80000000 | 51, 53 },
        }}),
        // Slot 'on_leftButtonBox_clicked'
        QtMocHelpers::SlotData<void(QAbstractButton *)>(54, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 55, 56 },
        }}),
        // Slot 'on_actionLegend_triggered'
        QtMocHelpers::SlotData<void(bool)>(57, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_actionLogScale_triggered'
        QtMocHelpers::SlotData<void(bool)>(59, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_actionCrosshairs_triggered'
        QtMocHelpers::SlotData<void(bool)>(60, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_actionTopAxis_triggered'
        QtMocHelpers::SlotData<void(bool)>(61, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_automaticUpdateCheckBox_toggled'
        QtMocHelpers::SlotData<void(bool)>(62, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_plotUat_currentItemChanged'
        QtMocHelpers::SlotData<void(const QModelIndex &, const QModelIndex &)>(63, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 15, 64 }, { 0x80000000 | 15, 65 },
        }}),
        // Slot 'on_actionGoToPacket_triggered'
        QtMocHelpers::SlotData<void()>(66, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_newToolButton_clicked'
        QtMocHelpers::SlotData<void()>(67, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_deleteToolButton_clicked'
        QtMocHelpers::SlotData<void()>(68, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_copyToolButton_clicked'
        QtMocHelpers::SlotData<void()>(69, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_clearToolButton_clicked'
        QtMocHelpers::SlotData<void()>(70, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_moveUpwardsToolButton_clicked'
        QtMocHelpers::SlotData<void()>(71, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_moveDownwardsToolButton_clicked'
        QtMocHelpers::SlotData<void()>(72, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_rightButtonBox_helpRequested'
        QtMocHelpers::SlotData<void()>(73, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionReset_triggered'
        QtMocHelpers::SlotData<void()>(74, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomIn_triggered'
        QtMocHelpers::SlotData<void()>(75, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomInX_triggered'
        QtMocHelpers::SlotData<void()>(76, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomInY_triggered'
        QtMocHelpers::SlotData<void()>(77, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomOut_triggered'
        QtMocHelpers::SlotData<void()>(78, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomOutX_triggered'
        QtMocHelpers::SlotData<void()>(79, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionZoomOutY_triggered'
        QtMocHelpers::SlotData<void()>(80, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveUp10_triggered'
        QtMocHelpers::SlotData<void()>(81, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveLeft10_triggered'
        QtMocHelpers::SlotData<void()>(82, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveRight10_triggered'
        QtMocHelpers::SlotData<void()>(83, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveDown10_triggered'
        QtMocHelpers::SlotData<void()>(84, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveUp1_triggered'
        QtMocHelpers::SlotData<void()>(85, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveLeft1_triggered'
        QtMocHelpers::SlotData<void()>(86, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveRight1_triggered'
        QtMocHelpers::SlotData<void()>(87, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveDown1_triggered'
        QtMocHelpers::SlotData<void()>(88, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveUp100_triggered'
        QtMocHelpers::SlotData<void()>(89, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveLeft100_triggered'
        QtMocHelpers::SlotData<void()>(90, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveRight100_triggered'
        QtMocHelpers::SlotData<void()>(91, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveDown100_triggered'
        QtMocHelpers::SlotData<void()>(92, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionToggleTimeOrigin_triggered'
        QtMocHelpers::SlotData<void()>(93, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_rightButtonBox_accepted'
        QtMocHelpers::SlotData<void()>(94, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionAutoScroll_triggered'
        QtMocHelpers::SlotData<void(bool)>(95, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_actionEnableMultiYAxes_triggered'
        QtMocHelpers::SlotData<void(bool)>(96, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 58 },
        }}),
        // Slot 'on_actionAddMarker_triggered'
        QtMocHelpers::SlotData<void()>(97, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionMoveMarker_triggered'
        QtMocHelpers::SlotData<void()>(98, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionShowPosMarker_triggered'
        QtMocHelpers::SlotData<void()>(99, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionShowMarkersDifference_triggered'
        QtMocHelpers::SlotData<void()>(100, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionDeleteMarker_triggered'
        QtMocHelpers::SlotData<void()>(101, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionDeleteAllMarkers_triggered'
        QtMocHelpers::SlotData<void()>(102, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'on_actionShowDataPointMarker_triggered'
        QtMocHelpers::SlotData<void()>(103, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<PlotDialog, qt_meta_tag_ZN10PlotDialogE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject PlotDialog::staticMetaObject = { {
    QMetaObject::SuperData::link<WiresharkDialog::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PlotDialogE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PlotDialogE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN10PlotDialogE_t>.metaTypes,
    nullptr
} };

void PlotDialog::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<PlotDialog *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->goToPacket((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 1: _t->updateMarker((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<double>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 2: _t->setPosMarker((*reinterpret_cast<std::add_pointer_t<double>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 3: _t->scheduleReplot(); break;
        case 4: _t->scheduleRecalc(); break;
        case 5: _t->scheduleRetap(); break;
        case 6: _t->modelDataChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[3]))); break;
        case 7: _t->modelRowsReset(); break;
        case 8: _t->modelRowsInserted((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 9: _t->modelRowsRemoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3]))); break;
        case 10: _t->modelRowsMoved((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[4])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[5]))); break;
        case 11: _t->onThemeChanged(); break;
        case 12: _t->applyChanges(); break;
        case 13: _t->updateStatistics(); break;
        case 14: _t->updateLegend(); break;
        case 15: _t->copyFromProfile((*reinterpret_cast<std::add_pointer_t<QString>>(_a[1]))); break;
        case 16: _t->copyAsCsvClicked(); break;
        case 17: _t->showContextMenu((*reinterpret_cast<std::add_pointer_t<QPoint>>(_a[1]))); break;
        case 18: _t->moveLegend(); break;
        case 19: _t->graphClicked((*reinterpret_cast<std::add_pointer_t<QMouseEvent*>>(_a[1]))); break;
        case 20: _t->mouseMoved((*reinterpret_cast<std::add_pointer_t<QMouseEvent*>>(_a[1]))); break;
        case 21: _t->mouseReleased((*reinterpret_cast<std::add_pointer_t<QMouseEvent*>>(_a[1]))); break;
        case 22: _t->selectedFrameChanged((*reinterpret_cast<std::add_pointer_t<QList<int>>>(_a[1]))); break;
        case 23: _t->plotUatSelectionChanged((*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QItemSelection>>(_a[2]))); break;
        case 24: _t->on_leftButtonBox_clicked((*reinterpret_cast<std::add_pointer_t<QAbstractButton*>>(_a[1]))); break;
        case 25: _t->on_actionLegend_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 26: _t->on_actionLogScale_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 27: _t->on_actionCrosshairs_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 28: _t->on_actionTopAxis_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 29: _t->on_automaticUpdateCheckBox_toggled((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 30: _t->on_plotUat_currentItemChanged((*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<QModelIndex>>(_a[2]))); break;
        case 31: _t->on_actionGoToPacket_triggered(); break;
        case 32: _t->on_newToolButton_clicked(); break;
        case 33: _t->on_deleteToolButton_clicked(); break;
        case 34: _t->on_copyToolButton_clicked(); break;
        case 35: _t->on_clearToolButton_clicked(); break;
        case 36: _t->on_moveUpwardsToolButton_clicked(); break;
        case 37: _t->on_moveDownwardsToolButton_clicked(); break;
        case 38: _t->on_rightButtonBox_helpRequested(); break;
        case 39: _t->on_actionReset_triggered(); break;
        case 40: _t->on_actionZoomIn_triggered(); break;
        case 41: _t->on_actionZoomInX_triggered(); break;
        case 42: _t->on_actionZoomInY_triggered(); break;
        case 43: _t->on_actionZoomOut_triggered(); break;
        case 44: _t->on_actionZoomOutX_triggered(); break;
        case 45: _t->on_actionZoomOutY_triggered(); break;
        case 46: _t->on_actionMoveUp10_triggered(); break;
        case 47: _t->on_actionMoveLeft10_triggered(); break;
        case 48: _t->on_actionMoveRight10_triggered(); break;
        case 49: _t->on_actionMoveDown10_triggered(); break;
        case 50: _t->on_actionMoveUp1_triggered(); break;
        case 51: _t->on_actionMoveLeft1_triggered(); break;
        case 52: _t->on_actionMoveRight1_triggered(); break;
        case 53: _t->on_actionMoveDown1_triggered(); break;
        case 54: _t->on_actionMoveUp100_triggered(); break;
        case 55: _t->on_actionMoveLeft100_triggered(); break;
        case 56: _t->on_actionMoveRight100_triggered(); break;
        case 57: _t->on_actionMoveDown100_triggered(); break;
        case 58: _t->on_actionToggleTimeOrigin_triggered(); break;
        case 59: _t->on_rightButtonBox_accepted(); break;
        case 60: _t->on_actionAutoScroll_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 61: _t->on_actionEnableMultiYAxes_triggered((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 62: _t->on_actionAddMarker_triggered(); break;
        case 63: _t->on_actionMoveMarker_triggered(); break;
        case 64: _t->on_actionShowPosMarker_triggered(); break;
        case 65: _t->on_actionShowMarkersDifference_triggered(); break;
        case 66: _t->on_actionDeleteMarker_triggered(); break;
        case 67: _t->on_actionDeleteAllMarkers_triggered(); break;
        case 68: _t->on_actionShowDataPointMarker_triggered(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 6:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 2:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 22:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QList<int> >(); break;
            }
            break;
        case 23:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 1:
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QItemSelection >(); break;
            }
            break;
        case 24:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAbstractButton* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (PlotDialog::*)(int , int )>(_a, &PlotDialog::goToPacket, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotDialog::*)(const int , const double , const int )>(_a, &PlotDialog::updateMarker, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (PlotDialog::*)(const double , const int , const int )>(_a, &PlotDialog::setPosMarker, 2))
            return;
    }
}

const QMetaObject *PlotDialog::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *PlotDialog::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN10PlotDialogE_t>.strings))
        return static_cast<void*>(this);
    return WiresharkDialog::qt_metacast(_clname);
}

int PlotDialog::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = WiresharkDialog::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 69)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 69;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 69)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 69;
    }
    return _id;
}

// SIGNAL 0
void PlotDialog::goToPacket(int _t1, int _t2)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 0, nullptr, _t1, _t2);
}

// SIGNAL 1
void PlotDialog::updateMarker(const int _t1, const double _t2, const int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 1, nullptr, _t1, _t2, _t3);
}

// SIGNAL 2
void PlotDialog::setPosMarker(const double _t1, const int _t2, const int _t3)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 2, nullptr, _t1, _t2, _t3);
}
QT_WARNING_POP
