/*
 * Do not modify this file. Changes will be overwritten.
 *
 * Generated automatically using "make-regs.py".
 */

#include <glib.h>
#include "wtap_modules.h"

const unsigned wtap_module_count = 74;

void register_5views(void);
void register_aethra(void);
void register_ascend(void);
void register_autosar_dlt(void);
void register_ber(void);
void register_blf(void);
void register_btsnoop(void);
void register_busmaster(void);
void register_camins(void);
void register_candump(void);
void register_canlogger(void);
void register_capsa(void);
void register_commview(void);
void register_cosine(void);
void register_csids(void);
void register_daintree_sna(void);
void register_darwin(void);
void register_dbs_etherwatch(void);
void register_dct2000(void);
void register_dct3trace(void);
void register_dpa400(void);
void register_ems(void);
void register_erf(void);
void register_eri_enb_log(void);
void register_eyesdn(void);
void register_hcidump(void);
void register_i4btrace(void);
void register_ipfix(void);
void register_iptrace(void);
void register_iseries(void);
void register_json(void);
void register_json_log(void);
void register_k12(void);
void register_k12text(void);
void register_lanalyzer(void);
void register_log3gpp(void);
void register_logcat(void);
void register_logcat_text(void);
void register_mime(void);
void register_mmodule(void);
void register_mp2t(void);
void register_mp4(void);
void register_mpeg(void);
void register_mplog(void);
void register_netlog(void);
void register_netmon(void);
void register_netscreen(void);
void register_nettl(void);
void register_nettrace_3gpp_32_423(void);
void register_netxray(void);
void register_nflx_custom(void);
void register_ngsniffer(void);
void register_nstrace(void);
void register_observer(void);
void register_packetlogger(void);
void register_peak_trc(void);
void register_peekclassic(void);
void register_peektagged(void);
void register_pppdump(void);
void register_procmon(void);
void register_radcom(void);
void register_rfc7468(void);
void register_rtpdump(void);
void register_ruby_marshal(void);
void register_snoop(void);
void register_stanag4607(void);
void register_sysdig(void);
void register_systemd_journal(void);
void register_tnef(void);
void register_toshiba(void);
void register_ttl(void);
void register_visual(void);
void register_vms(void);
void register_vwr(void);

wtap_module_reg_t const wtap_module_reg[] = {
    { "register_5views", register_5views },
    { "register_aethra", register_aethra },
    { "register_ascend", register_ascend },
    { "register_autosar_dlt", register_autosar_dlt },
    { "register_ber", register_ber },
    { "register_blf", register_blf },
    { "register_btsnoop", register_btsnoop },
    { "register_busmaster", register_busmaster },
    { "register_camins", register_camins },
    { "register_candump", register_candump },
    { "register_canlogger", register_canlogger },
    { "register_capsa", register_capsa },
    { "register_commview", register_commview },
    { "register_cosine", register_cosine },
    { "register_csids", register_csids },
    { "register_daintree_sna", register_daintree_sna },
    { "register_darwin", register_darwin },
    { "register_dbs_etherwatch", register_dbs_etherwatch },
    { "register_dct2000", register_dct2000 },
    { "register_dct3trace", register_dct3trace },
    { "register_dpa400", register_dpa400 },
    { "register_ems", register_ems },
    { "register_erf", register_erf },
    { "register_eri_enb_log", register_eri_enb_log },
    { "register_eyesdn", register_eyesdn },
    { "register_hcidump", register_hcidump },
    { "register_i4btrace", register_i4btrace },
    { "register_ipfix", register_ipfix },
    { "register_iptrace", register_iptrace },
    { "register_iseries", register_iseries },
    { "register_json", register_json },
    { "register_json_log", register_json_log },
    { "register_k12", register_k12 },
    { "register_k12text", register_k12text },
    { "register_lanalyzer", register_lanalyzer },
    { "register_log3gpp", register_log3gpp },
    { "register_logcat", register_logcat },
    { "register_logcat_text", register_logcat_text },
    { "register_mime", register_mime },
    { "register_mmodule", register_mmodule },
    { "register_mp2t", register_mp2t },
    { "register_mp4", register_mp4 },
    { "register_mpeg", register_mpeg },
    { "register_mplog", register_mplog },
    { "register_netlog", register_netlog },
    { "register_netmon", register_netmon },
    { "register_netscreen", register_netscreen },
    { "register_nettl", register_nettl },
    { "register_nettrace_3gpp_32_423", register_nettrace_3gpp_32_423 },
    { "register_netxray", register_netxray },
    { "register_nflx_custom", register_nflx_custom },
    { "register_ngsniffer", register_ngsniffer },
    { "register_nstrace", register_nstrace },
    { "register_observer", register_observer },
    { "register_packetlogger", register_packetlogger },
    { "register_peak_trc", register_peak_trc },
    { "register_peekclassic", register_peekclassic },
    { "register_peektagged", register_peektagged },
    { "register_pppdump", register_pppdump },
    { "register_procmon", register_procmon },
    { "register_radcom", register_radcom },
    { "register_rfc7468", register_rfc7468 },
    { "register_rtpdump", register_rtpdump },
    { "register_ruby_marshal", register_ruby_marshal },
    { "register_snoop", register_snoop },
    { "register_stanag4607", register_stanag4607 },
    { "register_sysdig", register_sysdig },
    { "register_systemd_journal", register_systemd_journal },
    { "register_tnef", register_tnef },
    { "register_toshiba", register_toshiba },
    { "register_ttl", register_ttl },
    { "register_visual", register_visual },
    { "register_vms", register_vms },
    { "register_vwr", register_vwr },
    { NULL, NULL }
};
