/*
 * data-midi-sysex-id.h
 *
 * This file was generated by running tools\dissector_generators\generate-midi-sysex-data.py .
 *
 * SPDX-License-Identifier: GPL-2.0-or-later
 */

#ifndef __PACKET_MIDI_SYSEX_ID_H__
#define __PACKET_MIDI_SYSEX_ID_H__

/* One-byte MIDI SysEx identifiers. */
#define MIDI_SYSEX_ID_INDICATOR_FOR_EXTENDED_MIDI_SYSEX_ID                    0x00
#define MIDI_SYSEX_ID_SEQUENTIAL_CIRCUITS                                     0x01
#define MIDI_SYSEX_ID_IDP                                                     0x02
#define MIDI_SYSEX_ID_VOYETRA_TURTLE_BEACH_INC                                0x03
#define MIDI_SYSEX_ID_MOOG_MUSIC                                              0x04
#define MIDI_SYSEX_ID_PASSPORT_DESIGNS                                        0x05
#define MIDI_SYSEX_ID_LEXICON_INC                                             0x06
#define MIDI_SYSEX_ID_KURZWEIL_YOUNG_CHANG                                    0x07
#define MIDI_SYSEX_ID_FENDER                                                  0x08
#define MIDI_SYSEX_ID_MIDI9                                                   0x09
#define MIDI_SYSEX_ID_AKG_ACOUSTICS                                           0x0A
#define MIDI_SYSEX_ID_VOYCE_MUSIC                                             0x0B
#define MIDI_SYSEX_ID_WAVEFRAME_TIMELINE                                      0x0C
#define MIDI_SYSEX_ID_ADA_SIGNAL_PROCESSORS_INC                               0x0D
#define MIDI_SYSEX_ID_GARFIELD_ELECTRONICS                                    0x0E
#define MIDI_SYSEX_ID_ENSONIQ                                                 0x0F
#define MIDI_SYSEX_ID_OBERHEIM                                                0x10
#define MIDI_SYSEX_ID_APPLE                                                   0x11
#define MIDI_SYSEX_ID_GREY_MATTER_RESPONSE                                    0x12
#define MIDI_SYSEX_ID_DIGIDESIGN_INC                                          0x13
#define MIDI_SYSEX_ID_PALMTREE_INSTRUMENTS                                    0x14
#define MIDI_SYSEX_ID_JLCOOPER_ELECTRONICS                                    0x15
#define MIDI_SYSEX_ID_LOWREY_ORGAN_COMPANY                                    0x16
#define MIDI_SYSEX_ID_ADAMS_SMITH                                             0x17
#define MIDI_SYSEX_ID_E_MU                                                    0x18
#define MIDI_SYSEX_ID_HARMONY_SYSTEMS                                         0x19
#define MIDI_SYSEX_ID_ART                                                     0x1A
#define MIDI_SYSEX_ID_BALDWIN                                                 0x1B
#define MIDI_SYSEX_ID_EVENTIDE                                                0x1C
#define MIDI_SYSEX_ID_INVENTRONICS                                            0x1D
#define MIDI_SYSEX_ID_KEY_CONCEPTS                                            0x1E
#define MIDI_SYSEX_ID_CLARITY                                                 0x1F
#define MIDI_SYSEX_ID_PASSAC                                                  0x20
#define MIDI_SYSEX_ID_PROEL_LABS_SIEL                                         0x21
#define MIDI_SYSEX_ID_SYNTHAXE_UK                                             0x22
#define MIDI_SYSEX_ID_STEPP                                                   0x23
#define MIDI_SYSEX_ID_HOHNER                                                  0x24
#define MIDI_SYSEX_ID_TWISTER                                                 0x25
#define MIDI_SYSEX_ID_KETRON_S_R_L                                            0x26
#define MIDI_SYSEX_ID_JELLINGHAUS_MS                                          0x27
#define MIDI_SYSEX_ID_SOUTHWORTH_MUSIC_SYSTEMS                                0x28
#define MIDI_SYSEX_ID_PPG_GERMANY                                             0x29
#define MIDI_SYSEX_ID_CESYG_LTD                                               0x2A
#define MIDI_SYSEX_ID_SOLID_STATE_LOGIC_ORGAN_SYSTEMS                         0x2B
#define MIDI_SYSEX_ID_AUDIO_VERITRIEB_P_STRUVEN                               0x2C
#define MIDI_SYSEX_ID_NEVE                                                    0x2D
#define MIDI_SYSEX_ID_SOUNDTRACS_LTD                                          0x2E
#define MIDI_SYSEX_ID_ELKA                                                    0x2F
#define MIDI_SYSEX_ID_DYNACORD                                                0x30
#define MIDI_SYSEX_ID_VISCOUNT_INTERNATIONAL_SPA_INTERCONTINENTAL_ELECTRONICS 0x31
#define MIDI_SYSEX_ID_DRAWMER                                                 0x32
#define MIDI_SYSEX_ID_CLAVIA_DIGITAL_INSTRUMENTS                              0x33
#define MIDI_SYSEX_ID_AUDIO_ARCHITECTURE                                      0x34
#define MIDI_SYSEX_ID_GENERALMUSIC_CORP_SPA                                   0x35
#define MIDI_SYSEX_ID_CHEETAH_MARKETING                                       0x36
#define MIDI_SYSEX_ID_C_T_M                                                   0x37
#define MIDI_SYSEX_ID_SIMMONS_UK                                              0x38
#define MIDI_SYSEX_ID_SOUNDCRAFT_ELECTRONICS                                  0x39
#define MIDI_SYSEX_ID_STEINBERG_MEDIA_TECHNOLOGIES_GMBH                       0x3A
#define MIDI_SYSEX_ID_WERSI_GMBH                                              0x3B
#define MIDI_SYSEX_ID_AVAB_NIETHAMMER_AB                                      0x3C
#define MIDI_SYSEX_ID_DIGIGRAM                                                0x3D
#define MIDI_SYSEX_ID_WALDORF_ELECTRONICS_GMBH                                0x3E
#define MIDI_SYSEX_ID_QUASIMIDI                                               0x3F
#define MIDI_SYSEX_ID_KAWAI_MUSICAL_INSTRUMENTS_MFG_CO_LTD                    0x40
#define MIDI_SYSEX_ID_ROLAND_CORPORATION                                      0x41
#define MIDI_SYSEX_ID_KORG_INC                                                0x42
#define MIDI_SYSEX_ID_YAMAHA_CORPORATION                                      0x43
#define MIDI_SYSEX_ID_CASIO_COMPUTER_CO_LTD                                   0x44
#define MIDI_SYSEX_ID_KAMIYA_STUDIO_CO_LTD                                    0x46
#define MIDI_SYSEX_ID_AKAI_ELECTRIC_CO_LTD                                    0x47
#define MIDI_SYSEX_ID_VICTOR_COMPANY_OF_JAPAN_LTD                             0x48
#define MIDI_SYSEX_ID_FUJITSU_LIMITED                                         0x4B
#define MIDI_SYSEX_ID_SONY_CORPORATION                                        0x4C
#define MIDI_SYSEX_ID_TEAC_CORPORATION                                        0x4E
#define MIDI_SYSEX_ID_MATSUSHITA_ELECTRIC_INDUSTRIAL_CO_LTD                   0x50
#define MIDI_SYSEX_ID_FOSTEX_CORPORATION                                      0x51
#define MIDI_SYSEX_ID_ZOOM_CORPORATION                                        0x52
#define MIDI_SYSEX_ID_MATSUSHITA_COMMUNICATION_INDUSTRIAL_CO_LTD              0x54
#define MIDI_SYSEX_ID_SUZUKI_MUSICAL_INSTRUMENTS_MFG_CO_LTD                   0x55
#define MIDI_SYSEX_ID_FUJI_SOUND_CORPORATION_LTD                              0x56
#define MIDI_SYSEX_ID_ACOUSTIC_TECHNICAL_LABORATORY_INC                       0x57
#define MIDI_SYSEX_ID_FAITH_INC                                               0x59
#define MIDI_SYSEX_ID_INTERNET_CORPORATION                                    0x5A
#define MIDI_SYSEX_ID_SEEKERS_CO_LTD                                          0x5C
#define MIDI_SYSEX_ID_SD_CARD_ASSOCIATION                                     0x5F
#define MIDI_SYSEX_ID_EDUCATIONAL_NON_COMMERCIAL_USE                          0x7D
#define MIDI_SYSEX_ID_NON_REAL_TIME_UNIVERSAL_SYSTEM_EXCLUSIVE                0x7E
#define MIDI_SYSEX_ID_REAL_TIME_UNIVERSAL_SYSTEM_EXCLUSIVE                    0x7F

/* Three-byte (extended) MIDI SysEx identifiers. */
#define MIDI_SYSEX_ID_TIME_WARNER_INTERACTIVE                                        0x000001
#define MIDI_SYSEX_ID_ADVANCED_GRAVIS_COMP_TECH_LTD                                  0x000002
#define MIDI_SYSEX_ID_MEDIA_VISION                                                   0x000003
#define MIDI_SYSEX_ID_DORNES_RESEARCH_GROUP                                          0x000004
#define MIDI_SYSEX_ID_K_MUSE                                                         0x000005
#define MIDI_SYSEX_ID_STYPHER                                                        0x000006
#define MIDI_SYSEX_ID_DIGITAL_MUSIC_CORP                                             0x000007
#define MIDI_SYSEX_ID_IOTA_SYSTEMS                                                   0x000008
#define MIDI_SYSEX_ID_NEW_ENGLAND_DIGITAL                                            0x000009
#define MIDI_SYSEX_ID_ARTISYN                                                        0x00000A
#define MIDI_SYSEX_ID_IVL_TECHNOLOGIES_LTD                                           0x00000B
#define MIDI_SYSEX_ID_SOUTHERN_MUSIC_SYSTEMS                                         0x00000C
#define MIDI_SYSEX_ID_LAKE_BUTLER_SOUND_COMPANY                                      0x00000D
#define MIDI_SYSEX_ID_ALESIS_STUDIO_ELECTRONICS                                      0x00000E
#define MIDI_SYSEX_ID_SOUND_CREATION                                                 0x00000F
#define MIDI_SYSEX_ID_DOD_ELECTRONICS_CORP                                           0x000010
#define MIDI_SYSEX_ID_STUDER_EDITECH                                                 0x000011
#define MIDI_SYSEX_ID_SONUS                                                          0x000012
#define MIDI_SYSEX_ID_TEMPORAL_ACUITY_PRODUCTS                                       0x000013
#define MIDI_SYSEX_ID_PERFECT_FRETWORKS                                              0x000014
#define MIDI_SYSEX_ID_KAT_INC                                                        0x000015
#define MIDI_SYSEX_ID_OPCODE_SYSTEMS                                                 0x000016
#define MIDI_SYSEX_ID_RANE_CORPORATION                                               0x000017
#define MIDI_SYSEX_ID_ANADI_ELECTRONIQUE                                             0x000018
#define MIDI_SYSEX_ID_KMX                                                            0x000019
#define MIDI_SYSEX_ID_ALLEN_HEATH_BRENELL                                            0x00001A
#define MIDI_SYSEX_ID_PEAVEY_ELECTRONICS                                             0x00001B
#define MIDI_SYSEX_ID_360_SYSTEMS                                                    0x00001C
#define MIDI_SYSEX_ID_SPECTRUM_DESIGN_AND_DEVELOPMENT                                0x00001D
#define MIDI_SYSEX_ID_MARQUIS_MUSIC                                                  0x00001E
#define MIDI_SYSEX_ID_ZETA_SYSTEMS                                                   0x00001F
#define MIDI_SYSEX_ID_AXXES_BRIAN_PARSONETT                                          0x000020
#define MIDI_SYSEX_ID_ORBAN                                                          0x000021
#define MIDI_SYSEX_ID_INDIAN_VALLEY_MFG                                              0x000022
#define MIDI_SYSEX_ID_TRITON                                                         0x000023
#define MIDI_SYSEX_ID_KTI                                                            0x000024
#define MIDI_SYSEX_ID_BREAKAWAY_TECHNOLOGIES                                         0x000025
#define MIDI_SYSEX_ID_LEPRECON_CAE_INC                                               0x000026
#define MIDI_SYSEX_ID_HARRISON_SYSTEMS_INC                                           0x000027
#define MIDI_SYSEX_ID_FUTURE_LAB_MARK_KUO                                            0x000028
#define MIDI_SYSEX_ID_ROCKTRON_CORPORATION                                           0x000029
#define MIDI_SYSEX_ID_PIANODISC_00002A                                               0x00002A
#define MIDI_SYSEX_ID_CANNON_RESEARCH_GROUP                                          0x00002B
#define MIDI_SYSEX_ID_RESERVED_00002C                                                0x00002C
#define MIDI_SYSEX_ID_RODGERS_INSTRUMENT_LLC                                         0x00002D
#define MIDI_SYSEX_ID_BLUE_SKY_LOGIC                                                 0x00002E
#define MIDI_SYSEX_ID_ENCORE_ELECTRONICS                                             0x00002F
#define MIDI_SYSEX_ID_UPTOWN                                                         0x000030
#define MIDI_SYSEX_ID_VOCE                                                           0x000031
#define MIDI_SYSEX_ID_CTI_AUDIO_INC_MUSICALLY_INTEL_DEVS                             0x000032
#define MIDI_SYSEX_ID_S3_INCORPORATED                                                0x000033
#define MIDI_SYSEX_ID_BRODERBUND_RED_ORB                                             0x000034
#define MIDI_SYSEX_ID_ALLEN_ORGAN_CO                                                 0x000035
#define MIDI_SYSEX_ID_RESERVED_000036                                                0x000036
#define MIDI_SYSEX_ID_MUSIC_QUEST                                                    0x000037
#define MIDI_SYSEX_ID_APHEX                                                          0x000038
#define MIDI_SYSEX_ID_GALLIEN_KRUEGER                                                0x000039
#define MIDI_SYSEX_ID_IBM                                                            0x00003A
#define MIDI_SYSEX_ID_MARK_OF_THE_UNICORN                                            0x00003B
#define MIDI_SYSEX_ID_HOTZ_CORPORATION                                               0x00003C
#define MIDI_SYSEX_ID_ETA_LIGHTING                                                   0x00003D
#define MIDI_SYSEX_ID_NSI_CORPORATION                                                0x00003E
#define MIDI_SYSEX_ID_AD_LIB_INC                                                     0x00003F
#define MIDI_SYSEX_ID_RICHMOND_SOUND_DESIGN                                          0x000040
#define MIDI_SYSEX_ID_MICROSOFT                                                      0x000041
#define MIDI_SYSEX_ID_MINDSCAPE_SOFTWARE_TOOLWORKS                                   0x000042
#define MIDI_SYSEX_ID_RUSS_JONES_MARKETING_NICHE                                     0x000043
#define MIDI_SYSEX_ID_INTONE                                                         0x000044
#define MIDI_SYSEX_ID_ADVANCED_REMOTE_TECHNOLOGIES                                   0x000045
#define MIDI_SYSEX_ID_WHITE_INSTRUMENTS                                              0x000046
#define MIDI_SYSEX_ID_GT_ELECTRONICS_GROOVE_TUBES                                    0x000047
#define MIDI_SYSEX_ID_PACIFIC_RESEARCH_ENGINEERING                                   0x000048
#define MIDI_SYSEX_ID_TIMELINE_VISTA_INC                                             0x000049
#define MIDI_SYSEX_ID_MESA_BOOGIE_LTD                                                0x00004A
#define MIDI_SYSEX_ID_FSLI                                                           0x00004B
#define MIDI_SYSEX_ID_SEQUOIA_DEVELOPMENT_GROUP                                      0x00004C
#define MIDI_SYSEX_ID_STUDIO_ELECTRONICS                                             0x00004D
#define MIDI_SYSEX_ID_EUPHONIX_INC                                                   0x00004E
#define MIDI_SYSEX_ID_INTERMIDI_INC                                                  0x00004F
#define MIDI_SYSEX_ID_MIDI_SOLUTIONS_INC                                             0x000050
#define MIDI_SYSEX_ID_3DO_COMPANY                                                    0x000051
#define MIDI_SYSEX_ID_LIGHTWAVE_RESEARCH_HIGH_END_SYSTEMS                            0x000052
#define MIDI_SYSEX_ID_MICRO_W_CORPORATION                                            0x000053
#define MIDI_SYSEX_ID_SPECTRAL_SYNTHESIS_INC                                         0x000054
#define MIDI_SYSEX_ID_LONE_WOLF                                                      0x000055
#define MIDI_SYSEX_ID_STUDIO_TECHNOLOGIES_INC                                        0x000056
#define MIDI_SYSEX_ID_PETERSON_ELECTRO_MUSICAL_PRODUCT_INC                           0x000057
#define MIDI_SYSEX_ID_ATARI_CORPORATION                                              0x000058
#define MIDI_SYSEX_ID_MARION_SYSTEMS_CORPORATION                                     0x000059
#define MIDI_SYSEX_ID_DESIGN_EVENT                                                   0x00005A
#define MIDI_SYSEX_ID_WINJAMMER_SOFTWARE_LTD                                         0x00005B
#define MIDI_SYSEX_ID_AT_T_BELL_LABORATORIES                                         0x00005C
#define MIDI_SYSEX_ID_RESERVED_00005D                                                0x00005D
#define MIDI_SYSEX_ID_SYMETRIX                                                       0x00005E
#define MIDI_SYSEX_ID_MIDI_THE_WORLD                                                 0x00005F
#define MIDI_SYSEX_ID_SPATIALIZER                                                    0x000060
#define MIDI_SYSEX_ID_MICROS_N_MIDI                                                  0x000061
#define MIDI_SYSEX_ID_ACCORDIANS_INTERNATIONAL                                       0x000062
#define MIDI_SYSEX_ID_EUPHONICS_NOW_3COM                                             0x000063
#define MIDI_SYSEX_ID_MUSONIX                                                        0x000064
#define MIDI_SYSEX_ID_TURTLE_BEACH_SYSTEMS_VOYETRA                                   0x000065
#define MIDI_SYSEX_ID_LOUD_TECHNOLOGIES_MACKIE                                       0x000066
#define MIDI_SYSEX_ID_COMPUSERVE                                                     0x000067
#define MIDI_SYSEX_ID_BEC_TECHNOLOGIES                                               0x000068
#define MIDI_SYSEX_ID_QRS_MUSIC_INC                                                  0x000069
#define MIDI_SYSEX_ID_P_G_MUSIC                                                      0x00006A
#define MIDI_SYSEX_ID_SIERRA_SEMICONDUCTOR                                           0x00006B
#define MIDI_SYSEX_ID_EPIGRAF                                                        0x00006C
#define MIDI_SYSEX_ID_ELECTRONICS_DIVERSIFIED_INC                                    0x00006D
#define MIDI_SYSEX_ID_TUNE_1000                                                      0x00006E
#define MIDI_SYSEX_ID_ADVANCED_MICRO_DEVICES                                         0x00006F
#define MIDI_SYSEX_ID_MEDIAMATION                                                    0x000070
#define MIDI_SYSEX_ID_SABINE_MUSICAL_MFG_CO_INC                                      0x000071
#define MIDI_SYSEX_ID_WOOG_LABS                                                      0x000072
#define MIDI_SYSEX_ID_MICROPOLIS_CORP                                                0x000073
#define MIDI_SYSEX_ID_TA_HORNG_MUSICAL_INSTRUMENT                                    0x000074
#define MIDI_SYSEX_ID_E_TEK_LABS_FORTE_TECH                                          0x000075
#define MIDI_SYSEX_ID_ELECTRO_VOICE                                                  0x000076
#define MIDI_SYSEX_ID_MIDISOFT_CORPORATION                                           0x000077
#define MIDI_SYSEX_ID_QSOUND_LABS                                                    0x000078
#define MIDI_SYSEX_ID_WESTREX                                                        0x000079
#define MIDI_SYSEX_ID_NVIDIA                                                         0x00007A
#define MIDI_SYSEX_ID_ESS_TECHNOLOGY                                                 0x00007B
#define MIDI_SYSEX_ID_MEDIA_TRIX_PERIPHERALS                                         0x00007C
#define MIDI_SYSEX_ID_BROOKTREE_CORP                                                 0x00007D
#define MIDI_SYSEX_ID_OTARI_CORP                                                     0x00007E
#define MIDI_SYSEX_ID_KEY_ELECTRONICS_INC                                            0x00007F
#define MIDI_SYSEX_ID_SHURE_INCORPORATED                                             0x000100
#define MIDI_SYSEX_ID_AURASOUND                                                      0x000101
#define MIDI_SYSEX_ID_CRYSTAL_SEMICONDUCTOR                                          0x000102
#define MIDI_SYSEX_ID_CONEXANT_ROCKWELL                                              0x000103
#define MIDI_SYSEX_ID_SILICON_GRAPHICS                                               0x000104
#define MIDI_SYSEX_ID_M_AUDIO_MIDIMAN                                                0x000105
#define MIDI_SYSEX_ID_PRESONUS                                                       0x000106
#define MIDI_SYSEX_ID_TOPAZ_ENTERPRISES                                              0x000108
#define MIDI_SYSEX_ID_CAST_LIGHTING                                                  0x000109
#define MIDI_SYSEX_ID_MICROSOFT_CONSUMER_DIVISION                                    0x00010A
#define MIDI_SYSEX_ID_SONIC_FOUNDRY                                                  0x00010B
#define MIDI_SYSEX_ID_LINE_6_FAST_FORWARD_YAMAHA                                     0x00010C
#define MIDI_SYSEX_ID_BEATNIK_INC                                                    0x00010D
#define MIDI_SYSEX_ID_VAN_KOEVERING_COMPANY                                          0x00010E
#define MIDI_SYSEX_ID_ALTECH_SYSTEMS                                                 0x00010F
#define MIDI_SYSEX_ID_S_S_RESEARCH                                                   0x000110
#define MIDI_SYSEX_ID_VLSI_TECHNOLOGY                                                0x000111
#define MIDI_SYSEX_ID_CHROMATIC_RESEARCH                                             0x000112
#define MIDI_SYSEX_ID_SAPPHIRE                                                       0x000113
#define MIDI_SYSEX_ID_IDRC                                                           0x000114
#define MIDI_SYSEX_ID_JUSTONIC_TUNING                                                0x000115
#define MIDI_SYSEX_ID_TORCOMP_RESEARCH_INC                                           0x000116
#define MIDI_SYSEX_ID_NEWTEK_INC                                                     0x000117
#define MIDI_SYSEX_ID_SOUND_SCULPTURE                                                0x000118
#define MIDI_SYSEX_ID_WALKER_TECHNICAL                                               0x000119
#define MIDI_SYSEX_ID_DIGITAL_HARMONY_PAVO                                           0x00011A
#define MIDI_SYSEX_ID_INVISION_INTERACTIVE                                           0x00011B
#define MIDI_SYSEX_ID_T_SQUARE_DESIGN                                                0x00011C
#define MIDI_SYSEX_ID_NEMESYS_MUSIC_TECHNOLOGY                                       0x00011D
#define MIDI_SYSEX_ID_DBX_PROFESSIONAL_HARMAN_INTL                                   0x00011E
#define MIDI_SYSEX_ID_SYNDYNE_CORPORATION                                            0x00011F
#define MIDI_SYSEX_ID_BITHEADZ                                                       0x000120
#define MIDI_SYSEX_ID_BANDLAB_TECHNOLOGIES                                           0x000121
#define MIDI_SYSEX_ID_ANALOG_DEVICES                                                 0x000122
#define MIDI_SYSEX_ID_NATIONAL_SEMICONDUCTOR                                         0x000123
#define MIDI_SYSEX_ID_BOOM_THEORY_ADINOLFI_ALTERNATIVE_PERCUSSION                    0x000124
#define MIDI_SYSEX_ID_VIRTUAL_DSP_CORPORATION                                        0x000125
#define MIDI_SYSEX_ID_ANTARES_SYSTEMS                                                0x000126
#define MIDI_SYSEX_ID_ANGEL_SOFTWARE                                                 0x000127
#define MIDI_SYSEX_ID_ST_LOUIS_MUSIC                                                 0x000128
#define MIDI_SYSEX_ID_PASSPORT_MUSIC_SOFTWARE_LLC_GVOX                               0x000129
#define MIDI_SYSEX_ID_ASHLEY_AUDIO_INC                                               0x00012A
#define MIDI_SYSEX_ID_VARI_LITE_INC                                                  0x00012B
#define MIDI_SYSEX_ID_SUMMIT_AUDIO_INC                                               0x00012C
#define MIDI_SYSEX_ID_AUREAL_SEMICONDUCTOR_INC                                       0x00012D
#define MIDI_SYSEX_ID_SEASOUND_LLC                                                   0x00012E
#define MIDI_SYSEX_ID_U_S_ROBOTICS                                                   0x00012F
#define MIDI_SYSEX_ID_AURISIS_RESEARCH                                               0x000130
#define MIDI_SYSEX_ID_NEARFIELD_RESEARCH                                             0x000131
#define MIDI_SYSEX_ID_FM7_INC                                                        0x000132
#define MIDI_SYSEX_ID_SWIVEL_SYSTEMS                                                 0x000133
#define MIDI_SYSEX_ID_HYPERACTIVE_AUDIO_SYSTEMS                                      0x000134
#define MIDI_SYSEX_ID_MIDILITE_CASTLE_STUDIOS_PRODUCTIONS                            0x000135
#define MIDI_SYSEX_ID_RADIKAL_TECHNOLOGIES                                           0x000136
#define MIDI_SYSEX_ID_ROGER_LINN_DESIGN                                              0x000137
#define MIDI_SYSEX_ID_TC_HELICON_VOCAL_TECHNOLOGIES                                  0x000138
#define MIDI_SYSEX_ID_EVENT_ELECTRONICS                                              0x000139
#define MIDI_SYSEX_ID_SONIC_NETWORK_INC                                              0x00013A
#define MIDI_SYSEX_ID_REALTIME_MUSIC_SOLUTIONS                                       0x00013B
#define MIDI_SYSEX_ID_APOGEE_DIGITAL                                                 0x00013C
#define MIDI_SYSEX_ID_CLASSICAL_ORGANS_INC                                           0x00013D
#define MIDI_SYSEX_ID_MICROTOOLS_INC                                                 0x00013E
#define MIDI_SYSEX_ID_NUMARK_INDUSTRIES                                              0x00013F
#define MIDI_SYSEX_ID_FRONTIER_DESIGN_GROUP_LLC                                      0x000140
#define MIDI_SYSEX_ID_RECORDARE_LLC                                                  0x000141
#define MIDI_SYSEX_ID_STARR_LABS                                                     0x000142
#define MIDI_SYSEX_ID_VOYAGER_SOUND_INC                                              0x000143
#define MIDI_SYSEX_ID_MANIFOLD_LABS                                                  0x000144
#define MIDI_SYSEX_ID_AVIOM_INC                                                      0x000145
#define MIDI_SYSEX_ID_MIXMEISTER_TECHNOLOGY                                          0x000146
#define MIDI_SYSEX_ID_NOTATION_SOFTWARE                                              0x000147
#define MIDI_SYSEX_ID_MERCURIAL_COMMUNICATIONS                                       0x000148
#define MIDI_SYSEX_ID_WAVE_ARTS                                                      0x000149
#define MIDI_SYSEX_ID_LOGIC_SEQUENCING_DEVICES                                       0x00014A
#define MIDI_SYSEX_ID_AXESS_ELECTRONICS                                              0x00014B
#define MIDI_SYSEX_ID_MUSE_RESEARCH                                                  0x00014C
#define MIDI_SYSEX_ID_OPEN_LABS                                                      0x00014D
#define MIDI_SYSEX_ID_GUILLEMOT_CORP                                                 0x00014E
#define MIDI_SYSEX_ID_SAMSON_TECHNOLOGIES                                            0x00014F
#define MIDI_SYSEX_ID_ELECTRONIC_THEATRE_CONTROLS                                    0x000150
#define MIDI_SYSEX_ID_BLACKBERRY_RIM                                                 0x000151
#define MIDI_SYSEX_ID_MOBILEER                                                       0x000152
#define MIDI_SYSEX_ID_SYNTHOGY                                                       0x000153
#define MIDI_SYSEX_ID_LYNX_STUDIO_TECHNOLOGY_INC                                     0x000154
#define MIDI_SYSEX_ID_DAMAGE_CONTROL_ENGINEERING_LLC                                 0x000155
#define MIDI_SYSEX_ID_YOST_ENGINEERING_INC                                           0x000156
#define MIDI_SYSEX_ID_BROOKS_FORSMAN_DESIGNS_LLC_DRUMLITE                            0x000157
#define MIDI_SYSEX_ID_INFINITE_RESPONSE                                              0x000158
#define MIDI_SYSEX_ID_GARRITAN_CORP                                                  0x000159
#define MIDI_SYSEX_ID_PLOGUE_ART_ET_TECHNOLOGIE_INC                                  0x00015A
#define MIDI_SYSEX_ID_RJM_MUSIC_TECHNOLOGY                                           0x00015B
#define MIDI_SYSEX_ID_CUSTOM_SOLUTIONS_SOFTWARE                                      0x00015C
#define MIDI_SYSEX_ID_SONARCANA_LLC_HIGHLY_LIQUID                                    0x00015D
#define MIDI_SYSEX_ID_CENTRANCE                                                      0x00015E
#define MIDI_SYSEX_ID_KESUMO_LLC                                                     0x00015F
#define MIDI_SYSEX_ID_STANTON_GIBSON_BRANDS                                          0x000160
#define MIDI_SYSEX_ID_LIVID_INSTRUMENTS                                              0x000161
#define MIDI_SYSEX_ID_FIRST_ACT_745_MEDIA                                            0x000162
#define MIDI_SYSEX_ID_PYGRAPHICS_INC                                                 0x000163
#define MIDI_SYSEX_ID_PANADIGM_INNOVATIONS_LTD                                       0x000164
#define MIDI_SYSEX_ID_AVEDIS_ZILDJIAN_CO                                             0x000165
#define MIDI_SYSEX_ID_AUVITAL_MUSIC_CORP                                             0x000166
#define MIDI_SYSEX_ID_YOU_ROCK_GUITAR_WAS_INSPIRED_INSTRUMENTS                       0x000167
#define MIDI_SYSEX_ID_CHRIS_GRIGG_DESIGNS                                            0x000168
#define MIDI_SYSEX_ID_SLATE_DIGITAL_LLC                                              0x000169
#define MIDI_SYSEX_ID_MIXWARE                                                        0x00016A
#define MIDI_SYSEX_ID_SOCIAL_ENTROPY                                                 0x00016B
#define MIDI_SYSEX_ID_SOURCE_AUDIO_LLC                                               0x00016C
#define MIDI_SYSEX_ID_ERNIE_BALL_MUSIC_MAN                                           0x00016D
#define MIDI_SYSEX_ID_FISHMAN                                                        0x00016E
#define MIDI_SYSEX_ID_CUSTOM_AUDIO_ELECTRONICS                                       0x00016F
#define MIDI_SYSEX_ID_AMERICAN_AUDIO_DJ                                              0x000170
#define MIDI_SYSEX_ID_MEGA_CONTROL_SYSTEMS                                           0x000171
#define MIDI_SYSEX_ID_KILPATRICK_AUDIO                                               0x000172
#define MIDI_SYSEX_ID_ICONNECTIVITY                                                  0x000173
#define MIDI_SYSEX_ID_FRACTAL_AUDIO                                                  0x000174
#define MIDI_SYSEX_ID_NETLOGIC_MICROSYSTEMS                                          0x000175
#define MIDI_SYSEX_ID_MUSIC_COMPUTING                                                0x000176
#define MIDI_SYSEX_ID_NEKTAR_TECHNOLOGY_INC                                          0x000177
#define MIDI_SYSEX_ID_ZENPH_SOUND_INNOVATIONS                                        0x000178
#define MIDI_SYSEX_ID_DJTECHTOOLS_COM                                                0x000179
#define MIDI_SYSEX_ID_REZONANCE_LABS                                                 0x00017A
#define MIDI_SYSEX_ID_DECIBEL_ELEVEN                                                 0x00017B
#define MIDI_SYSEX_ID_CNMAT                                                          0x00017C
#define MIDI_SYSEX_ID_MEDIA_OVERKILL                                                 0x00017D
#define MIDI_SYSEX_ID_CONFUSION_STUDIOS                                              0x00017E
#define MIDI_SYSEX_ID_MOFORTE_INC                                                    0x00017F
#define MIDI_SYSEX_ID_MISELU_INC                                                     0x000200
#define MIDI_SYSEX_ID_AMELIAS_COMPASS_LLC                                            0x000201
#define MIDI_SYSEX_ID_ZIVIX_LLC                                                      0x000202
#define MIDI_SYSEX_ID_ARTIPHON                                                       0x000203
#define MIDI_SYSEX_ID_SYNCLAVIER_DIGITAL                                             0x000204
#define MIDI_SYSEX_ID_LIGHT_SOUND_CONTROL_DEVICES_LLC                                0x000205
#define MIDI_SYSEX_ID_RETRONYMS_INC                                                  0x000206
#define MIDI_SYSEX_ID_JS_TECHNOLOGIES                                                0x000207
#define MIDI_SYSEX_ID_QUICCO_SOUND                                                   0x000208
#define MIDI_SYSEX_ID_A_DESIGNS_AUDIO                                                0x000209
#define MIDI_SYSEX_ID_MCCARTHY_MUSIC_CORP                                            0x00020A
#define MIDI_SYSEX_ID_DENON_DJ                                                       0x00020B
#define MIDI_SYSEX_ID_KEITH_ROBERT_MURRAY                                            0x00020C
#define MIDI_SYSEX_ID_GOOGLE                                                         0x00020D
#define MIDI_SYSEX_ID_ISP_TECHNOLOGIES                                               0x00020E
#define MIDI_SYSEX_ID_ABSTRAKT_INSTRUMENTS_LLC                                       0x00020F
#define MIDI_SYSEX_ID_MERIS_LLC                                                      0x000210
#define MIDI_SYSEX_ID_SENSORPOINT_LLC                                                0x000211
#define MIDI_SYSEX_ID_HI_Z_LABS                                                      0x000212
#define MIDI_SYSEX_ID_IMITONE                                                        0x000213
#define MIDI_SYSEX_ID_INTELLIJEL_DESIGNS_INC                                         0x000214
#define MIDI_SYSEX_ID_DASZ_INSTRUMENTS_INC                                           0x000215
#define MIDI_SYSEX_ID_REMIDI                                                         0x000216
#define MIDI_SYSEX_ID_DISASTER_AREA_DESIGNS_LLC                                      0x000217
#define MIDI_SYSEX_ID_UNIVERSAL_AUDIO                                                0x000218
#define MIDI_SYSEX_ID_CARTER_DUNCAN_CORP                                             0x000219
#define MIDI_SYSEX_ID_ESSENTIAL_TECHNOLOGY_00021A                                    0x00021A
#define MIDI_SYSEX_ID_CANTUX_RESEARCH_LLC                                            0x00021B
#define MIDI_SYSEX_ID_HUMMEL_TECHNOLOGIES                                            0x00021C
#define MIDI_SYSEX_ID_SENSEL_INC                                                     0x00021D
#define MIDI_SYSEX_ID_DBML_GROUP                                                     0x00021E
#define MIDI_SYSEX_ID_MADRONA_LABS                                                   0x00021F
#define MIDI_SYSEX_ID_MESA_BOOGIE                                                    0x000220
#define MIDI_SYSEX_ID_EFFIGY_LABS                                                    0x000221
#define MIDI_SYSEX_ID_AMENOTE                                                        0x000222
#define MIDI_SYSEX_ID_RED_PANDA_LLC                                                  0x000223
#define MIDI_SYSEX_ID_ONSONG_LLC                                                     0x000224
#define MIDI_SYSEX_ID_JAMBOXX_INC                                                    0x000225
#define MIDI_SYSEX_ID_ELECTRO_HARMONIX                                               0x000226
#define MIDI_SYSEX_ID_RND64_INC                                                      0x000227
#define MIDI_SYSEX_ID_NEUNABER_TECHNOLOGY_LLC                                        0x000228
#define MIDI_SYSEX_ID_KAOM_INC                                                       0x000229
#define MIDI_SYSEX_ID_HALLOWELL_EMC                                                  0x00022A
#define MIDI_SYSEX_ID_SOUND_DEVICES_LLC                                              0x00022B
#define MIDI_SYSEX_ID_SPECTRASONICS_INC                                              0x00022C
#define MIDI_SYSEX_ID_SECOND_SOUND_LLC                                               0x00022D
#define MIDI_SYSEX_ID_8EO_HORN                                                       0x00022E
#define MIDI_SYSEX_ID_VIDVOX_LLC                                                     0x00022F
#define MIDI_SYSEX_ID_MATTHEWS_EFFECTS                                               0x000230
#define MIDI_SYSEX_ID_BRIGHT_BLUE_BEETLE                                             0x000231
#define MIDI_SYSEX_ID_AUDIO_IMPRESSIONS                                              0x000232
#define MIDI_SYSEX_ID_LOOPERLATIVE                                                   0x000233
#define MIDI_SYSEX_ID_STEINWAY                                                       0x000234
#define MIDI_SYSEX_ID_INGENIOUS_ARTS_AND_TECHNOLOGIES_LLC                            0x000235
#define MIDI_SYSEX_ID_DCA_AUDIO                                                      0x000236
#define MIDI_SYSEX_ID_BUCHLA_USA                                                     0x000237
#define MIDI_SYSEX_ID_SINICON                                                        0x000238
#define MIDI_SYSEX_ID_ISLA_INSTRUMENTS                                               0x000239
#define MIDI_SYSEX_ID_SOUNDIRON_LLC                                                  0x00023A
#define MIDI_SYSEX_ID_SONOCLAST_LLC                                                  0x00023B
#define MIDI_SYSEX_ID_COPPER_AND_CEDAR                                               0x00023C
#define MIDI_SYSEX_ID_WHIRLED_NOTES                                                  0x00023D
#define MIDI_SYSEX_ID_CEJETVOLE_LLC                                                  0x00023E
#define MIDI_SYSEX_ID_DAWN_AUDIO_LLC                                                 0x00023F
#define MIDI_SYSEX_ID_SPACE_BRAIN_CIRCUITS                                           0x000240
#define MIDI_SYSEX_ID_CAEDENCE                                                       0x000241
#define MIDI_SYSEX_ID_HCN_DESIGNS_LLC_THE_MIDI_MAKER                                 0x000242
#define MIDI_SYSEX_ID_PTZOPTICS                                                      0x000243
#define MIDI_SYSEX_ID_NOISE_ENGINEERING                                              0x000244
#define MIDI_SYSEX_ID_SYNTHESIA_LLC                                                  0x000245
#define MIDI_SYSEX_ID_JEFF_WHITEHEAD_LUTHERIE_LLC                                    0x000246
#define MIDI_SYSEX_ID_WAMPLER_PEDALS_INC                                             0x000247
#define MIDI_SYSEX_ID_TAPIS_MAGIQUE                                                  0x000248
#define MIDI_SYSEX_ID_LEAF_SECRETS                                                   0x000249
#define MIDI_SYSEX_ID_GROOVE_SYNTHESIS_00024A                                        0x00024A
#define MIDI_SYSEX_ID_AUDIOCIPHER_TECHNOLOGIES_LLC                                   0x00024B
#define MIDI_SYSEX_ID_MELLOTRON_INC                                                  0x00024C
#define MIDI_SYSEX_ID_HOLOGRAM_ELECTRONICS_LLC                                       0x00024D
#define MIDI_SYSEX_ID_ICON_AMERICAS_LLC                                              0x00024E
#define MIDI_SYSEX_ID_SINGULAR_SOUND                                                 0x00024F
#define MIDI_SYSEX_ID_GENOVATION_INC                                                 0x000250
#define MIDI_SYSEX_ID_METHOD_RED                                                     0x000251
#define MIDI_SYSEX_ID_BRAIN_INVENTIONS                                               0x000252
#define MIDI_SYSEX_ID_SYNERVOZ_COMMUNICATIONS_INC                                    0x000253
#define MIDI_SYSEX_ID_HYPERTRIANGLE_INC                                              0x000254
#define MIDI_SYSEX_ID_DIGIBRASS_LLC                                                  0x000255
#define MIDI_SYSEX_ID_MIDI2_MARKETING_LLC                                            0x000256
#define MIDI_SYSEX_ID_TAQS_IM                                                        0x000257
#define MIDI_SYSEX_ID_CSS_DESIGNS                                                    0x000258
#define MIDI_SYSEX_ID_MOZAIC_BEATS                                                   0x000259
#define MIDI_SYSEX_ID_PIANODISC_00025A                                               0x00025A
#define MIDI_SYSEX_ID_UWYN                                                           0x00025B
#define MIDI_SYSEX_ID_FSK_AUDIO                                                      0x00025C
#define MIDI_SYSEX_ID_ETERNAL_RESEARCH_LLC                                           0x00025D
#define MIDI_SYSEX_ID_AZOTEQ_INC                                                     0x00025E
#define MIDI_SYSEX_ID_HY_MUSIC                                                       0x00025F
#define MIDI_SYSEX_ID_SOUND_MAGIC_CO_LTD                                             0x000260
#define MIDI_SYSEX_ID_DIRTYWAVE                                                      0x000261
#define MIDI_SYSEX_ID_KINOTONE                                                       0x000262
#define MIDI_SYSEX_ID_YODER_PRECISION_INSTRUMENTS                                    0x000263
#define MIDI_SYSEX_ID_MCLAREN_LABS                                                   0x000264
#define MIDI_SYSEX_ID_GIGPERFOMER                                                    0x000265
#define MIDI_SYSEX_ID_GROOVE_SYNTHESIS_000266                                        0x000266
#define MIDI_SYSEX_ID_STELLR_AUDIO                                                   0x000268
#define MIDI_SYSEX_ID_ESSENTIAL_TECHNOLOGY_000269                                    0x000269
#define MIDI_SYSEX_ID_LOOKC_MUSIC                                                    0x00026A
#define MIDI_SYSEX_ID_HOMEY_MUSIC                                                    0x00026B
#define MIDI_SYSEX_ID_LORRE_MILL                                                     0x00026C
#define MIDI_SYSEX_ID_PLAYTIME_ENGINEERING                                           0x00026D
#define MIDI_SYSEX_ID_PIPEORGAN_TECH                                                 0x00026E
#define MIDI_SYSEX_ID_SEER_SYSTEMS                                                   0x00026F
#define MIDI_SYSEX_ID_AM_COMPONENTS_LLC                                              0x000270
#define MIDI_SYSEX_ID_MUSIC_TECHNOLOGIES_GROUP                                       0x000271
#define MIDI_SYSEX_ID_GLX_AUDIO_CO                                                   0x000272
#define MIDI_SYSEX_ID_FREE_SOFTWARE_FOUNDATION_INC                                   0x000273
#define MIDI_SYSEX_ID_DREAM_SAS                                                      0x002000
#define MIDI_SYSEX_ID_STRAND_LIGHTING                                                0x002001
#define MIDI_SYSEX_ID_AMEK_DIV_OF_HARMAN_INDUSTRIES                                  0x002002
#define MIDI_SYSEX_ID_CASA_DI_RISPARMIO_DI_LORETO                                    0x002003
#define MIDI_SYSEX_ID_BOHM_ELECTRONIC_GMBH                                           0x002004
#define MIDI_SYSEX_ID_SYNTEC_DIGITAL_AUDIO                                           0x002005
#define MIDI_SYSEX_ID_TRIDENT_AUDIO_DEVELOPMENTS                                     0x002006
#define MIDI_SYSEX_ID_REAL_WORLD_STUDIO                                              0x002007
#define MIDI_SYSEX_ID_EVOLUTION_SYNTHESIS_LTD                                        0x002008
#define MIDI_SYSEX_ID_YES_TECHNOLOGY                                                 0x002009
#define MIDI_SYSEX_ID_AUDIOMATICA                                                    0x00200A
#define MIDI_SYSEX_ID_BONTEMPI_SPA_SIGMA                                             0x00200B
#define MIDI_SYSEX_ID_F_B_T_ELETTRONICA_SPA                                          0x00200C
#define MIDI_SYSEX_ID_MIDITEMP_GMBH                                                  0x00200D
#define MIDI_SYSEX_ID_LA_AUDIO_LARKING_AUDIO                                         0x00200E
#define MIDI_SYSEX_ID_ZERO_88_LIGHTING_LIMITED                                       0x00200F
#define MIDI_SYSEX_ID_MICON_AUDIO_ELECTRONICS_GMBH                                   0x002010
#define MIDI_SYSEX_ID_FOREFRONT_TECHNOLOGY                                           0x002011
#define MIDI_SYSEX_ID_STUDIO_AUDIO_AND_VIDEO_LTD                                     0x002012
#define MIDI_SYSEX_ID_KENTON_ELECTRONICS                                             0x002013
#define MIDI_SYSEX_ID_CELCO_ELECTROSONIC                                             0x002014
#define MIDI_SYSEX_ID_ADB                                                            0x002015
#define MIDI_SYSEX_ID_MARSHALL_PRODUCTS_LIMITED                                      0x002016
#define MIDI_SYSEX_ID_DDA                                                            0x002017
#define MIDI_SYSEX_ID_BSS_AUDIO_LTD                                                  0x002018
#define MIDI_SYSEX_ID_MA_LIGHTING_TECHNOLOGY                                         0x002019
#define MIDI_SYSEX_ID_FATAR_SRL_C_O_MUSIC_INDUSTRIES                                 0x00201A
#define MIDI_SYSEX_ID_QSC_AUDIO_PRODUCTS_INC                                         0x00201B
#define MIDI_SYSEX_ID_ARTISAN_CLASIC_ORGAN_INC                                       0x00201C
#define MIDI_SYSEX_ID_ORLA_SPA                                                       0x00201D
#define MIDI_SYSEX_ID_PINNACLE_AUDIO_KLARK_TEKNIK_PLC                                0x00201E
#define MIDI_SYSEX_ID_TC_ELECTRONICS                                                 0x00201F
#define MIDI_SYSEX_ID_DOEPFER_MUSIKELEKTRONIK_GMBH                                   0x002020
#define MIDI_SYSEX_ID_CREATIVE_ATC_E_MU                                              0x002021
#define MIDI_SYSEX_ID_SEYDDO_MINAMI                                                  0x002022
#define MIDI_SYSEX_ID_LG_ELECTRONICS_GOLDSTAR                                        0x002023
#define MIDI_SYSEX_ID_MIDISOFT_SAS_DI_M_CIMA_C                                       0x002024
#define MIDI_SYSEX_ID_SAMICK_MUSICAL_INST_CO_LTD                                     0x002025
#define MIDI_SYSEX_ID_PENNY_AND_GILES_BOWTHORPE_PLC                                  0x002026
#define MIDI_SYSEX_ID_ACORN_COMPUTER                                                 0x002027
#define MIDI_SYSEX_ID_LSC_ELECTRONICS_PTY_LTD                                        0x002028
#define MIDI_SYSEX_ID_FOCUSRITE_NOVATION                                             0x002029
#define MIDI_SYSEX_ID_SAMKYUNG_MECHATRONICS                                          0x00202A
#define MIDI_SYSEX_ID_MEDELI_ELECTRONICS_CO                                          0x00202B
#define MIDI_SYSEX_ID_CHARLIE_LAB_SRL                                                0x00202C
#define MIDI_SYSEX_ID_BLUE_CHIP_MUSIC_TECHNOLOGY                                     0x00202D
#define MIDI_SYSEX_ID_BEE_OH_CORP                                                    0x00202E
#define MIDI_SYSEX_ID_LG_SEMICON_AMERICA                                             0x00202F
#define MIDI_SYSEX_ID_TESI                                                           0x002030
#define MIDI_SYSEX_ID_EMAGIC                                                         0x002031
#define MIDI_SYSEX_ID_BEHRINGER_GMBH                                                 0x002032
#define MIDI_SYSEX_ID_ACCESS_MUSIC_ELECTRONICS                                       0x002033
#define MIDI_SYSEX_ID_SYNOPTIC                                                       0x002034
#define MIDI_SYSEX_ID_HANMESOFT                                                      0x002035
#define MIDI_SYSEX_ID_TERRATEC_ELECTRONIC_GMBH                                       0x002036
#define MIDI_SYSEX_ID_PROEL_SPA                                                      0x002037
#define MIDI_SYSEX_ID_IBK_MIDI                                                       0x002038
#define MIDI_SYSEX_ID_IRCAM                                                          0x002039
#define MIDI_SYSEX_ID_PROPELLERHEAD_SOFTWARE                                         0x00203A
#define MIDI_SYSEX_ID_RED_SOUND_SYSTEMS_LTD                                          0x00203B
#define MIDI_SYSEX_ID_ELEKTRON_ESI_AB                                                0x00203C
#define MIDI_SYSEX_ID_SINTEFEX_AUDIO                                                 0x00203D
#define MIDI_SYSEX_ID_MAM_MUSIC_AND_MORE                                             0x00203E
#define MIDI_SYSEX_ID_AMSARO_GMBH                                                    0x00203F
#define MIDI_SYSEX_ID_CDS_ADVANCED_TECHNOLOGY_BV_LANBOX                              0x002040
#define MIDI_SYSEX_ID_MODE_MACHINES_TOUCHED_BY_SOUND_GMBH                            0x002041
#define MIDI_SYSEX_ID_DSP_ARTS                                                       0x002042
#define MIDI_SYSEX_ID_PHIL_REES_MUSIC_TECH                                           0x002043
#define MIDI_SYSEX_ID_STAMER_MUSIKANLAGEN_GMBH                                       0x002044
#define MIDI_SYSEX_ID_MUSICAL_MUNTANER_S_A_DBA_SOUNDART                              0x002045
#define MIDI_SYSEX_ID_C_MEXX_SOFTWARE                                                0x002046
#define MIDI_SYSEX_ID_KLAVIS_TECHNOLOGIES                                            0x002047
#define MIDI_SYSEX_ID_NOTEHEADS_AB                                                   0x002048
#define MIDI_SYSEX_ID_ALGORITHMIX                                                    0x002049
#define MIDI_SYSEX_ID_SKRYDSTRUP_R_D                                                 0x00204A
#define MIDI_SYSEX_ID_PROFESSIONAL_AUDIO_COMPANY                                     0x00204B
#define MIDI_SYSEX_ID_NEWWAVE_LABS_MADWAVES                                          0x00204C
#define MIDI_SYSEX_ID_VERMONA                                                        0x00204D
#define MIDI_SYSEX_ID_NOKIA                                                          0x00204E
#define MIDI_SYSEX_ID_WAVE_IDEA                                                      0x00204F
#define MIDI_SYSEX_ID_HARTMANN_GMBH                                                  0x002050
#define MIDI_SYSEX_ID_LIONS_TRACS                                                    0x002051
#define MIDI_SYSEX_ID_ANALOGUE_SYSTEMS                                               0x002052
#define MIDI_SYSEX_ID_FOCAL_JMLAB                                                    0x002053
#define MIDI_SYSEX_ID_RINGWAY_ELECTRONICS_CHANG_ZHOU_CO_LTD                          0x002054
#define MIDI_SYSEX_ID_FAITH_TECHNOLOGIES_DIGIPLUG                                    0x002055
#define MIDI_SYSEX_ID_SHOWWORKS                                                      0x002056
#define MIDI_SYSEX_ID_MANIKIN_ELECTRONIC                                             0x002057
#define MIDI_SYSEX_ID_1_COME_TECH                                                    0x002058
#define MIDI_SYSEX_ID_PHONIC_CORP                                                    0x002059
#define MIDI_SYSEX_ID_DOLBY_AUSTRALIA_LAKE                                           0x00205A
#define MIDI_SYSEX_ID_SILANSYS_TECHNOLOGIES                                          0x00205B
#define MIDI_SYSEX_ID_WINBOND_ELECTRONICS                                            0x00205C
#define MIDI_SYSEX_ID_CINETIX_MEDIEN_UND_INTERFACE_GMBH                              0x00205D
#define MIDI_SYSEX_ID_A_G_SOLUZIONI_DIGITALI                                         0x00205E
#define MIDI_SYSEX_ID_SEQUENTIX_GMBH                                                 0x00205F
#define MIDI_SYSEX_ID_ORAM_PRO_AUDIO                                                 0x002060
#define MIDI_SYSEX_ID_BE4_LTD                                                        0x002061
#define MIDI_SYSEX_ID_INFECTION_MUSIC                                                0x002062
#define MIDI_SYSEX_ID_CENTRAL_MUSIC_CO_CME                                           0x002063
#define MIDI_SYSEX_ID_GENOQS_MACHINES_GMBH                                           0x002064
#define MIDI_SYSEX_ID_MEDIALON                                                       0x002065
#define MIDI_SYSEX_ID_WAVES_AUDIO_LTD                                                0x002066
#define MIDI_SYSEX_ID_JERASH_LABS                                                    0x002067
#define MIDI_SYSEX_ID_DA_FACT                                                        0x002068
#define MIDI_SYSEX_ID_ELBY_DESIGNS                                                   0x002069
#define MIDI_SYSEX_ID_SPECTRAL_AUDIO                                                 0x00206A
#define MIDI_SYSEX_ID_ARTURIA                                                        0x00206B
#define MIDI_SYSEX_ID_VIXID                                                          0x00206C
#define MIDI_SYSEX_ID_C_THRU_MUSIC                                                   0x00206D
#define MIDI_SYSEX_ID_YA_HORNG_ELECTRONIC_CO_LTD                                     0x00206E
#define MIDI_SYSEX_ID_SM_PRO_AUDIO                                                   0x00206F
#define MIDI_SYSEX_ID_OTO_MACHINES                                                   0x002070
#define MIDI_SYSEX_ID_ELZAB_S_A_G_LAB                                                0x002071
#define MIDI_SYSEX_ID_BLACKSTAR_AMPLIFICATION_LTD                                    0x002072
#define MIDI_SYSEX_ID_M3I_TECHNOLOGIES_GMBH                                          0x002073
#define MIDI_SYSEX_ID_GEMALTO_FROM_XIRING                                            0x002074
#define MIDI_SYSEX_ID_PROSTAGE_SL                                                    0x002075
#define MIDI_SYSEX_ID_TEENAGE_ENGINEERING                                            0x002076
#define MIDI_SYSEX_ID_TOBIAS_ERICHSEN_CONSULTING                                     0x002077
#define MIDI_SYSEX_ID_NIXER_LTD                                                      0x002078
#define MIDI_SYSEX_ID_HANPIN_ELECTRON_CO_LTD                                         0x002079
#define MIDI_SYSEX_ID_MIDI_HARDWARE_R_SOWA                                           0x00207A
#define MIDI_SYSEX_ID_BEYOND_MUSIC_INDUSTRIAL_LTD                                    0x00207B
#define MIDI_SYSEX_ID_KISS_BOX_B_V                                                   0x00207C
#define MIDI_SYSEX_ID_MISA_DIGITAL_TECHNOLOGIES_LTD                                  0x00207D
#define MIDI_SYSEX_ID_AI_MUSICS_TECHNOLOGY_INC                                       0x00207E
#define MIDI_SYSEX_ID_SERATO_INC_LP                                                  0x00207F
#define MIDI_SYSEX_ID_LIMEX                                                          0x002100
#define MIDI_SYSEX_ID_KYODDAY_TOKAI                                                  0x002101
#define MIDI_SYSEX_ID_MUTABLE_INSTRUMENTS                                            0x002102
#define MIDI_SYSEX_ID_PRESONUS_SOFTWARE_LTD                                          0x002103
#define MIDI_SYSEX_ID_INGENICO_WAS_XIRING                                            0x002104
#define MIDI_SYSEX_ID_FAIRLIGHT_INSTRUMENTS_PTY_LTD                                  0x002105
#define MIDI_SYSEX_ID_MUSICOM_LAB                                                    0x002106
#define MIDI_SYSEX_ID_MODAL_ELECTRONICS_MODULUS_VACOLOCO                             0x002107
#define MIDI_SYSEX_ID_RWA_HONG_KONG_LIMITED                                          0x002108
#define MIDI_SYSEX_ID_NATIVE_INSTRUMENTS                                             0x002109
#define MIDI_SYSEX_ID_NAONEXT                                                        0x00210A
#define MIDI_SYSEX_ID_MFB                                                            0x00210B
#define MIDI_SYSEX_ID_TEKNEL_RESEARCH                                                0x00210C
#define MIDI_SYSEX_ID_PLOYTEC_GMBH                                                   0x00210D
#define MIDI_SYSEX_ID_SURFIN_KANGAROO_STUDIO                                         0x00210E
#define MIDI_SYSEX_ID_PHILIPS_ELECTRONICS_HK_LTD                                     0x00210F
#define MIDI_SYSEX_ID_ROLI_LTD                                                       0x002110
#define MIDI_SYSEX_ID_PANDA_AUDIO_LTD                                                0x002111
#define MIDI_SYSEX_ID_BAUM_SOFTWARE                                                  0x002112
#define MIDI_SYSEX_ID_MACHINEWERKS_LTD                                               0x002113
#define MIDI_SYSEX_ID_XIAMEN_ELANE_ELECTRONICS                                       0x002114
#define MIDI_SYSEX_ID_MARSHALL_AMPLIFICATION_PLC                                     0x002115
#define MIDI_SYSEX_ID_KIWITECHNICS_LTD                                               0x002116
#define MIDI_SYSEX_ID_ROB_PAPEN                                                      0x002117
#define MIDI_SYSEX_ID_SPICETONE_OU                                                   0x002118
#define MIDI_SYSEX_ID_V3SOUND                                                        0x002119
#define MIDI_SYSEX_ID_IK_MULTIMEDIA                                                  0x00211A
#define MIDI_SYSEX_ID_NOVALIA_LTD                                                    0x00211B
#define MIDI_SYSEX_ID_MODOR_MUSIC                                                    0x00211C
#define MIDI_SYSEX_ID_ABLETON                                                        0x00211D
#define MIDI_SYSEX_ID_DTRONICS                                                       0x00211E
#define MIDI_SYSEX_ID_ZAQ_AUDIO                                                      0x00211F
#define MIDI_SYSEX_ID_MUABAOBAO_EDUCATION_TECHNOLOGY_CO_LTD                          0x002120
#define MIDI_SYSEX_ID_FLUX_EFFECTS                                                   0x002121
#define MIDI_SYSEX_ID_AUDIOTHINGIES_MCDA                                             0x002122
#define MIDI_SYSEX_ID_RETROKITS                                                      0x002123
#define MIDI_SYSEX_ID_MORNINGSTAR_FX_PTE_LTD                                         0x002124
#define MIDI_SYSEX_ID_CHANGSHA_HOTONE_AUDIO_CO_LTD                                   0x002125
#define MIDI_SYSEX_ID_EXPRESSIVE_E                                                   0x002126
#define MIDI_SYSEX_ID_EXPERT_SLEEPERS_LTD                                            0x002127
#define MIDI_SYSEX_ID_TIMECODE_VISION_TECHNOLOGY                                     0x002128
#define MIDI_SYSEX_ID_HORNBERG_RESEARCH_GBR                                          0x002129
#define MIDI_SYSEX_ID_SONIC_POTIONS                                                  0x00212A
#define MIDI_SYSEX_ID_AUDIOFRONT                                                     0x00212B
#define MIDI_SYSEX_ID_FREDS_LAB                                                      0x00212C
#define MIDI_SYSEX_ID_AUDIO_MODELING                                                 0x00212D
#define MIDI_SYSEX_ID_C_BECHSTEIN_DIGITAL_GMBH                                       0x00212E
#define MIDI_SYSEX_ID_MOTAS_ELECTRONICS_LTD                                          0x00212F
#define MIDI_SYSEX_ID_ELK_AUDIO                                                      0x002130
#define MIDI_SYSEX_ID_SONIC_ACADEMY_LTD                                              0x002131
#define MIDI_SYSEX_ID_BOME_SOFTWARE                                                  0x002132
#define MIDI_SYSEX_ID_AODYO_SAS                                                      0x002133
#define MIDI_SYSEX_ID_PIANOFORCE_S_R_O                                               0x002134
#define MIDI_SYSEX_ID_DREADBOX_P_C                                                   0x002135
#define MIDI_SYSEX_ID_TOUCHKEYS_INSTRUMENTS_LTD                                      0x002136
#define MIDI_SYSEX_ID_THE_GIGRIG_LTD                                                 0x002137
#define MIDI_SYSEX_ID_ALM_CO                                                         0x002138
#define MIDI_SYSEX_ID_CH_SOUND_DESIGN                                                0x002139
#define MIDI_SYSEX_ID_BEAT_BARS                                                      0x00213A
#define MIDI_SYSEX_ID_BLOKAS                                                         0x00213B
#define MIDI_SYSEX_ID_GEWA_MUSIC_GMBH                                                0x00213C
#define MIDI_SYSEX_ID_DADAMACHINES                                                   0x00213D
#define MIDI_SYSEX_ID_AUGMENTED_INSTRUMENTS_LTD_BELA                                 0x00213E
#define MIDI_SYSEX_ID_SUPERCRITICAL_LTD                                              0x00213F
#define MIDI_SYSEX_ID_GENKI_INSTRUMENTS                                              0x002140
#define MIDI_SYSEX_ID_MARIENBERG_DEVICES_GERMANY                                     0x002141
#define MIDI_SYSEX_ID_SUPPERWARE_LTD                                                 0x002142
#define MIDI_SYSEX_ID_IMOXPLUS_BVBA                                                  0x002143
#define MIDI_SYSEX_ID_SWAPP_TECHNOLOGIES_SRL                                         0x002144
#define MIDI_SYSEX_ID_ELECTRA_ONE_S_R_O                                              0x002145
#define MIDI_SYSEX_ID_DIGITAL_CLEF_LIMITED                                           0x002146
#define MIDI_SYSEX_ID_PAUL_WHITTINGTON_GROUP_LTD                                     0x002147
#define MIDI_SYSEX_ID_MUSIC_HACKSPACE                                                0x002148
#define MIDI_SYSEX_ID_BITWIG_GMBH                                                    0x002149
#define MIDI_SYSEX_ID_ENHANCIA                                                       0x00214A
#define MIDI_SYSEX_ID_KV_331                                                         0x00214B
#define MIDI_SYSEX_ID_TEHNICADELARTE                                                 0x00214C
#define MIDI_SYSEX_ID_ENDLESSS_STUDIO                                                0x00214D
#define MIDI_SYSEX_ID_DONGGUAN_MIDIPLUS_CO_LTD                                       0x00214E
#define MIDI_SYSEX_ID_GRACELY_PTY_LTD                                                0x00214F
#define MIDI_SYSEX_ID_EMBODME                                                        0x002150
#define MIDI_SYSEX_ID_MUSESCORE                                                      0x002151
#define MIDI_SYSEX_ID_EPFL_E_LAB                                                     0x002152
#define MIDI_SYSEX_ID_ORB3_LTD                                                       0x002153
#define MIDI_SYSEX_ID_PITCH_INNOVATIONS                                              0x002154
#define MIDI_SYSEX_ID_PLAYCES                                                        0x002155
#define MIDI_SYSEX_ID_UDO_AUDIO_LTD                                                  0x002156
#define MIDI_SYSEX_ID_RSS_SOUND_DESIGN                                               0x002157
#define MIDI_SYSEX_ID_NONLINEAR_LABS_GMBH                                            0x002158
#define MIDI_SYSEX_ID_ROBKOO_INFORMATION_TECHNOLOGIES_CO_LTD                         0x002159
#define MIDI_SYSEX_ID_CARI_ELECTRONIC                                                0x00215A
#define MIDI_SYSEX_ID_OXI_ELECTRONIC_INSTRUMENTS_SL                                  0x00215B
#define MIDI_SYSEX_ID_XMPT                                                           0x00215C
#define MIDI_SYSEX_ID_SHANGHAI_HUAXIN_MUSICAL_INSTRUMENT                             0x00215D
#define MIDI_SYSEX_ID_SHENZHEN_HUASHI_TECHNOLOGY_CO_LTD                              0x00215E
#define MIDI_SYSEX_ID_GUANGZHOU_RANTION_TECHNOLOGY_CO_LTD                            0x002160
#define MIDI_SYSEX_ID_RYME_MUSIC                                                     0x002161
#define MIDI_SYSEX_ID_GS_MUSIC                                                       0x002162
#define MIDI_SYSEX_ID_SHENZHEN_FLAMMA_INNOVATION_CO_LTD                              0x002163
#define MIDI_SYSEX_ID_SHENZHEN_MOOER_AUDIO_CO_LTD                                    0x002164
#define MIDI_SYSEX_ID_RAW_MATERIAL_SOFTWARE_LIMITED_JUCE                             0x002165
#define MIDI_SYSEX_ID_BIRDKIDS                                                       0x002166
#define MIDI_SYSEX_ID_BEIJING_QIANYINHULIAN_TECH_CO                                  0x002167
#define MIDI_SYSEX_ID_NIMIKRY_MUSIC_OG                                               0x002168
#define MIDI_SYSEX_ID_NEWZIK                                                         0x002169
#define MIDI_SYSEX_ID_HAMBURG_WAVE                                                   0x00216A
#define MIDI_SYSEX_ID_GRIMM_AUDIO                                                    0x00216B
#define MIDI_SYSEX_ID_ARCANA_INSTRUMENTS_LTD                                         0x00216C
#define MIDI_SYSEX_ID_GAMECHANGER_AUDIO                                              0x00216D
#define MIDI_SYSEX_ID_OAKTONE                                                        0x00216E
#define MIDI_SYSEX_ID_THE_DIGI_GURDY_A_MIDI_HURDY_GURDY                              0x00216F
#define MIDI_SYSEX_ID_MUSIKRAKEN                                                     0x002170
#define MIDI_SYSEX_ID_PHOTOSYNTH_INTERFACE                                           0x002171
#define MIDI_SYSEX_ID_INSTRUMENTS_OF_THINGS                                          0x002172
#define MIDI_SYSEX_ID_OODI                                                           0x002173
#define MIDI_SYSEX_ID_KOMIRES_SP_Z_O_O                                               0x002174
#define MIDI_SYSEX_ID_LEHLE_GMBH                                                     0x002175
#define MIDI_SYSEX_ID_JOUE_MUSIC_INSTRUMENTS                                         0x002176
#define MIDI_SYSEX_ID_GUANGZHOU_PEARL_RIVER_AMASON_DIGITAL_MUSICAL_INSTRUMENT_CO_LTD 0x002177
#define MIDI_SYSEX_ID_RHESUS_ENGINEERING_GMBH                                        0x002178
#define MIDI_SYSEX_ID_BREMMERS_AUDIO_DESIGN                                          0x002179
#define MIDI_SYSEX_ID_CHERUB_TECHNOLOGY_CO_LTD                                       0x00217A
#define MIDI_SYSEX_ID_SYNTHSTROM_AUDIBLE                                             0x00217B
#define MIDI_SYSEX_ID_NEURAL_DSP_TECHNOLOGIES_OY                                     0x00217C
#define MIDI_SYSEX_ID_2BOX_AB                                                        0x00217D
#define MIDI_SYSEX_ID_INTUITIVE_INSTRUMENTS                                          0x00217E
#define MIDI_SYSEX_ID_TWISTED_ELECTRONS                                              0x00217F
#define MIDI_SYSEX_ID_WILDCARD_ENGINEERING_BV                                        0x002200
#define MIDI_SYSEX_ID_DATO_MUSICAL_INSTRUMENTS                                       0x002201
#define MIDI_SYSEX_ID_JTJ_AUDIO                                                      0x002202
#define MIDI_SYSEX_ID_MELBOURNE_INSTRUMENTS                                          0x002203
#define MIDI_SYSEX_ID_AZMINO                                                         0x002204
#define MIDI_SYSEX_ID_TELEMIDI_TRIGITAL_PTY_LTD                                      0x002205
#define MIDI_SYSEX_ID_TYLIUM                                                         0x002206
#define MIDI_SYSEX_ID_ENGINEERING_LAB                                                0x002207
#define MIDI_SYSEX_ID_ARCHAEA_MODULAR_SYNTHESIS_LTD                                  0x002208
#define MIDI_SYSEX_ID_TENTACLE_SYNC_GMBH                                             0x002209
#define MIDI_SYSEX_ID_MISA_DIGITAL_PTY_LTD                                           0x00220A
#define MIDI_SYSEX_ID_HEAVY_PROCRASTINATION_INDUSTRIES_APS                           0x00220B
#define MIDI_SYSEX_ID_TELEPATHIC_PTY_LTD                                             0x00220C
#define MIDI_SYSEX_ID_NEOHARP_LLC_FZ                                                 0x00220D
#define MIDI_SYSEX_ID_RHODES_MUSIC_GROUP                                             0x00220E
#define MIDI_SYSEX_ID_HALBESTUNDE_GMBH                                               0x00220F
#define MIDI_SYSEX_ID_FLUQE                                                          0x002211
#define MIDI_SYSEX_ID_BLACKMAGIC_DESIGN_PTY_LTD                                      0x002212
#define MIDI_SYSEX_ID_TANGIBLE_WAVES                                                 0x002213
#define MIDI_SYSEX_ID_HUEBNER_INFORMATIONSELEKTRONIK                                 0x002214
#define MIDI_SYSEX_ID_ADDICTIVE_INSTRUMENTS                                          0x002215
#define MIDI_SYSEX_ID_LIGHTREFT_KK                                                   0x002216
#define MIDI_SYSEX_ID_MORPHOR                                                        0x002217
#define MIDI_SYSEX_ID_FILISIA_INTERFACES_LTD                                         0x002218
#define MIDI_SYSEX_ID_NEUZEIT_INSTRUMENTS                                            0x002219
#define MIDI_SYSEX_ID_KOMA_ELEKTRONIK                                                0x00221A
#define MIDI_SYSEX_ID_FRESHWATER_INSTRUMENTS                                         0x00221B
#define MIDI_SYSEX_ID_FULL_FAT                                                       0x00221C
#define MIDI_SYSEX_ID_GOOROO_CONTROLLERS                                             0x00221D
#define MIDI_SYSEX_ID_JK                                                             0x00221E
#define MIDI_SYSEX_ID_SHENZHEN_DALE_SENSOR_TECH_CO_LTD                               0x00221F
#define MIDI_SYSEX_ID_ANTESCOFO                                                      0x002220
#define MIDI_SYSEX_ID_CRIMSON_TECHNOLOGY_INC                                         0x004000
#define MIDI_SYSEX_ID_SOFTBANK_MOBILE_CORP                                           0x004001
#define MIDI_SYSEX_ID_D_M_HOLDINGS_INC                                               0x004003
#define MIDI_SYSEX_ID_XING_INC                                                       0x004004
#define MIDI_SYSEX_ID_ALPHATHETA_CORPORATION                                         0x004005
#define MIDI_SYSEX_ID_PIONEER_CORPORATION                                            0x004006
#define MIDI_SYSEX_ID_SLIK_CORPORATION                                               0x004007
#define MIDI_SYSEX_ID_SIGBOOST_INC                                                   0x004800
#define MIDI_SYSEX_ID_LOST_TECHNOLOGY                                                0x004801
#define MIDI_SYSEX_ID_FAN_WIND_GOD                                                   0x004802
#define MIDI_SYSEX_ID_TSUKUBA_SCIENTIFIC_CO_LTD                                      0x004803
#define MIDI_SYSEX_ID_SONICWARE_CO_LTD                                               0x004804
#define MIDI_SYSEX_ID_POPPY_SEED_STUDIO                                              0x004805
#define MIDI_SYSEX_ID_BLACK_CORPORATION_LLC                                          0x004806
#define MIDI_SYSEX_ID_G_TONE_GIKEN_CO_LTD                                            0x004807
#define MIDI_SYSEX_ID_NITROPLASMA                                                    0x004808

extern value_string_ext midi_sysex_id_vals_ext;
extern value_string_ext midi_sysex_extended_id_vals_ext;

#endif /* __PACKET_MIDI_SYSEX_ID_H__ */
