/******************************************************************************
** Copyright (C) 2014 ascolab GmbH. All Rights Reserved.
** Web: http://www.ascolab.com
**
** SPDX-License-Identifier: GPL-2.0-or-later
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** Project: OpcUa Wireshark Plugin
**
** Author: Hannes Mezger <hannes.mezger@ascolab.com>
**
******************************************************************************/

/** @brief Array of strings representing the different status codes. */
extern const value_string g_statusCodes[];
