/********************************************************************************
** Form generated from reading UI file 'welcome_page.ui'
**
** Created by: Qt User Interface Compiler version 6.11.1
**
** WARNING! All changes made in this file will be lost when recompiling UI file!
********************************************************************************/

#ifndef UI_WELCOME_PAGE_H
#define UI_WELCOME_PAGE_H

#include <QtCore/QVariant>
#include <QtWidgets/QApplication>
#include <QtWidgets/QHBoxLayout>
#include <QtWidgets/QListView>
#include <QtWidgets/QSpacerItem>
#include <QtWidgets/QVBoxLayout>
#include <QtWidgets/QWidget>
#include "widgets/capture_card_widget.h"
#include "widgets/clickable_label.h"
#include "widgets/info_banner_widget.h"
#include "widgets/learn_card_widget.h"
#include "widgets/welcome_header_widget.h"

QT_BEGIN_NAMESPACE

class Ui_WelcomePage
{
public:
    QVBoxLayout *pageLayout;
    WelcomeHeaderWidget *welcomeHeaderWidget;
    QHBoxLayout *contentAreaLayout;
    QWidget *mainContentContainer;
    QVBoxLayout *mainContentLayout;
    QVBoxLayout *openFileSectionLayout;
    ClickableLabel *openFileSectionLabel;
    QListView *openFileSectionRecentList;
    CaptureCardWidget *captureSectionCard;
    QWidget *sidebarContainer;
    QVBoxLayout *sidebarLayout;
    InfoBannerWidget *tipsSectionCard;
    LearnCardWidget *learnSectionCard;
    QSpacerItem *sidebarSpacer;

    void setupUi(QWidget *WelcomePage)
    {
        if (WelcomePage->objectName().isEmpty())
            WelcomePage->setObjectName("WelcomePage");
        WelcomePage->resize(920, 663);
        pageLayout = new QVBoxLayout(WelcomePage);
        pageLayout->setSpacing(0);
        pageLayout->setObjectName("pageLayout");
        pageLayout->setContentsMargins(0, 0, 0, 0);
        welcomeHeaderWidget = new WelcomeHeaderWidget(WelcomePage);
        welcomeHeaderWidget->setObjectName("welcomeHeaderWidget");
        QSizePolicy sizePolicy(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::Fixed);
        sizePolicy.setHorizontalStretch(0);
        sizePolicy.setVerticalStretch(0);
        sizePolicy.setHeightForWidth(welcomeHeaderWidget->sizePolicy().hasHeightForWidth());
        welcomeHeaderWidget->setSizePolicy(sizePolicy);

        pageLayout->addWidget(welcomeHeaderWidget);

        contentAreaLayout = new QHBoxLayout();
        contentAreaLayout->setSpacing(12);
        contentAreaLayout->setObjectName("contentAreaLayout");
        contentAreaLayout->setContentsMargins(24, 16, 24, 16);
        mainContentContainer = new QWidget(WelcomePage);
        mainContentContainer->setObjectName("mainContentContainer");
        QSizePolicy sizePolicy1(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::Preferred);
        sizePolicy1.setHorizontalStretch(1);
        sizePolicy1.setVerticalStretch(0);
        sizePolicy1.setHeightForWidth(mainContentContainer->sizePolicy().hasHeightForWidth());
        mainContentContainer->setSizePolicy(sizePolicy1);
        mainContentContainer->setMinimumSize(QSize(400, 0));
        mainContentLayout = new QVBoxLayout(mainContentContainer);
        mainContentLayout->setObjectName("mainContentLayout");
        mainContentLayout->setContentsMargins(0, 0, 0, 0);
        openFileSectionLayout = new QVBoxLayout();
        openFileSectionLayout->setObjectName("openFileSectionLayout");
        openFileSectionLabel = new ClickableLabel(mainContentContainer);
        openFileSectionLabel->setObjectName("openFileSectionLabel");
        QSizePolicy sizePolicy2(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Preferred);
        sizePolicy2.setHorizontalStretch(0);
        sizePolicy2.setVerticalStretch(0);
        sizePolicy2.setHeightForWidth(openFileSectionLabel->sizePolicy().hasHeightForWidth());
        openFileSectionLabel->setSizePolicy(sizePolicy2);

        openFileSectionLayout->addWidget(openFileSectionLabel);

        openFileSectionRecentList = new QListView(mainContentContainer);
        openFileSectionRecentList->setObjectName("openFileSectionRecentList");
        QSizePolicy sizePolicy3(QSizePolicy::Policy::Expanding, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy3.setHorizontalStretch(0);
        sizePolicy3.setVerticalStretch(1);
        sizePolicy3.setHeightForWidth(openFileSectionRecentList->sizePolicy().hasHeightForWidth());
        openFileSectionRecentList->setSizePolicy(sizePolicy3);
        openFileSectionRecentList->setMinimumSize(QSize(0, 120));
        openFileSectionRecentList->setHorizontalScrollBarPolicy(Qt::ScrollBarPolicy::ScrollBarAlwaysOff);

        openFileSectionLayout->addWidget(openFileSectionRecentList);


        mainContentLayout->addLayout(openFileSectionLayout);

        captureSectionCard = new CaptureCardWidget(mainContentContainer);
        captureSectionCard->setObjectName("captureSectionCard");
        QSizePolicy sizePolicy4(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy4.setHorizontalStretch(0);
        sizePolicy4.setVerticalStretch(0);
        sizePolicy4.setHeightForWidth(captureSectionCard->sizePolicy().hasHeightForWidth());
        captureSectionCard->setSizePolicy(sizePolicy4);
        captureSectionCard->setFrameShape(QFrame::Shape::NoFrame);

        mainContentLayout->addWidget(captureSectionCard);


        contentAreaLayout->addWidget(mainContentContainer);

        sidebarContainer = new QWidget(WelcomePage);
        sidebarContainer->setObjectName("sidebarContainer");
        sizePolicy2.setHeightForWidth(sidebarContainer->sizePolicy().hasHeightForWidth());
        sidebarContainer->setSizePolicy(sizePolicy2);
        sidebarContainer->setMinimumSize(QSize(300, 0));
        sidebarContainer->setMaximumSize(QSize(300, 16777215));
        sidebarLayout = new QVBoxLayout(sidebarContainer);
        sidebarLayout->setSpacing(16);
        sidebarLayout->setObjectName("sidebarLayout");
        sidebarLayout->setContentsMargins(0, 0, 0, 0);
        tipsSectionCard = new InfoBannerWidget(sidebarContainer);
        tipsSectionCard->setObjectName("tipsSectionCard");
        QSizePolicy sizePolicy5(QSizePolicy::Policy::Fixed, QSizePolicy::Policy::Fixed);
        sizePolicy5.setHorizontalStretch(0);
        sizePolicy5.setVerticalStretch(0);
        sizePolicy5.setHeightForWidth(tipsSectionCard->sizePolicy().hasHeightForWidth());
        tipsSectionCard->setSizePolicy(sizePolicy5);
        tipsSectionCard->setFrameShape(QFrame::Shape::NoFrame);

        sidebarLayout->addWidget(tipsSectionCard);

        learnSectionCard = new LearnCardWidget(sidebarContainer);
        learnSectionCard->setObjectName("learnSectionCard");
        QSizePolicy sizePolicy6(QSizePolicy::Policy::Preferred, QSizePolicy::Policy::MinimumExpanding);
        sizePolicy6.setHorizontalStretch(0);
        sizePolicy6.setVerticalStretch(1);
        sizePolicy6.setHeightForWidth(learnSectionCard->sizePolicy().hasHeightForWidth());
        learnSectionCard->setSizePolicy(sizePolicy6);
        learnSectionCard->setFrameShape(QFrame::Shape::StyledPanel);

        sidebarLayout->addWidget(learnSectionCard);

        sidebarSpacer = new QSpacerItem(0, 0, QSizePolicy::Policy::Minimum, QSizePolicy::Policy::Expanding);

        sidebarLayout->addItem(sidebarSpacer);


        contentAreaLayout->addWidget(sidebarContainer);


        pageLayout->addLayout(contentAreaLayout);


        retranslateUi(WelcomePage);

        QMetaObject::connectSlotsByName(WelcomePage);
    } // setupUi

    void retranslateUi(QWidget *WelcomePage)
    {
        WelcomePage->setWindowTitle(QCoreApplication::translate("WelcomePage", "Form", nullptr));
#if QT_CONFIG(tooltip)
        openFileSectionLabel->setToolTip(QCoreApplication::translate("WelcomePage", "<html><head/><body><p>Open a file on your file system</p></body></html>", nullptr));
#endif // QT_CONFIG(tooltip)
#if QT_CONFIG(accessibility)
        openFileSectionLabel->setAccessibleName(QCoreApplication::translate("WelcomePage", "Open", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        openFileSectionLabel->setAccessibleDescription(QCoreApplication::translate("WelcomePage", "List of recently opened files. Files that can no longer be found on disk are shown in italics.", nullptr));
#endif // QT_CONFIG(accessibility)
        openFileSectionLabel->setText(QCoreApplication::translate("WelcomePage", "<h2>Open</h2>", nullptr));
#if QT_CONFIG(accessibility)
        openFileSectionRecentList->setAccessibleName(QCoreApplication::translate("WelcomePage", "Recent capture files", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        openFileSectionRecentList->setAccessibleDescription(QCoreApplication::translate("WelcomePage", "Capture files that have been opened previously", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        tipsSectionCard->setAccessibleName(QCoreApplication::translate("WelcomePage", "Tips and announcements", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        tipsSectionCard->setAccessibleDescription(QCoreApplication::translate("WelcomePage", "Rotating tips, sponsor acknowledgements, and announcements for Wireshark and Stratoshark conferences.", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        learnSectionCard->setAccessibleName(QCoreApplication::translate("WelcomePage", "Learn", nullptr));
#endif // QT_CONFIG(accessibility)
#if QT_CONFIG(accessibility)
        learnSectionCard->setAccessibleDescription(QCoreApplication::translate("WelcomePage", "Links to documentation, wiki, and community resources.", nullptr));
#endif // QT_CONFIG(accessibility)
    } // retranslateUi

};

namespace Ui {
    class WelcomePage: public Ui_WelcomePage {};
} // namespace Ui

QT_END_NAMESPACE

#endif // UI_WELCOME_PAGE_H
