/****************************************************************************
** Meta object code from reading C++ file 'hex_data_source_view.h'
**
** Created by: The Qt Meta Object Compiler version 69 (Qt 6.11.1)
**
** WARNING! All changes made in this file will be lost!
*****************************************************************************/

#include "../../../../../ui/qt/widgets/hex_data_source_view.h"
#include <QtGui/qtextcursor.h>
#include <QtCore/qmetatype.h>

#include <QtCore/qtmochelpers.h>

#include <memory>


#include <QtCore/qxptype_traits.h>
#if !defined(Q_MOC_OUTPUT_REVISION)
#error "The header file 'hex_data_source_view.h' doesn't include <QObject>."
#elif Q_MOC_OUTPUT_REVISION != 69
#error "This file was generated using the moc from 6.11.1. It"
#error "cannot be used with the include files from this version of Qt."
#error "(The moc has changed too much.)"
#endif

#ifndef Q_CONSTINIT
#define Q_CONSTINIT
#endif

QT_WARNING_PUSH
QT_WARNING_DISABLE_DEPRECATED
QT_WARNING_DISABLE_GCC("-Wuseless-cast")
namespace {
struct qt_meta_tag_ZN17HexDataSourceViewE_t {};
} // unnamed namespace

template <> constexpr inline auto HexDataSourceView::qt_create_metaobjectdata<qt_meta_tag_ZN17HexDataSourceViewE_t>()
{
    namespace QMC = QtMocConstants;
    QtMocHelpers::StringRefStorage qt_stringData {
        "HexDataSourceView",
        "byteViewSettingsChanged",
        "",
        "addAnnotationRequested",
        "editAnnotationRequested",
        "removeAnnotationRequested",
        "offsetStartRequested",
        "byte",
        "offsetEndRequested",
        "offsetMarkersCleared",
        "setMonospaceFont",
        "QFont",
        "mono_font",
        "updateByteViewSettings",
        "markProtocol",
        "start",
        "length",
        "markField",
        "scroll_to",
        "hover",
        "markAppendix",
        "unmarkField",
        "copyBytes",
        "unused",
        "setHexDisplayFormat",
        "QAction*",
        "action",
        "setCharacterEncoding",
        "toggleHoverAllowed",
        "allowed",
        "requestAddAnnotation",
        "requestEditAnnotation",
        "requestRemoveAnnotation",
        "requestSetOffsetStart",
        "requestSetOffsetEnd",
        "requestClearOffsetMarkers"
    };

    QtMocHelpers::UintData qt_methods {
        // Signal 'byteViewSettingsChanged'
        QtMocHelpers::SignalData<void()>(1, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'addAnnotationRequested'
        QtMocHelpers::SignalData<void()>(3, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'editAnnotationRequested'
        QtMocHelpers::SignalData<void()>(4, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'removeAnnotationRequested'
        QtMocHelpers::SignalData<void()>(5, 2, QMC::AccessPublic, QMetaType::Void),
        // Signal 'offsetStartRequested'
        QtMocHelpers::SignalData<void(int)>(6, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'offsetEndRequested'
        QtMocHelpers::SignalData<void(int)>(8, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 7 },
        }}),
        // Signal 'offsetMarkersCleared'
        QtMocHelpers::SignalData<void()>(9, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'setMonospaceFont'
        QtMocHelpers::SlotData<void(const QFont &)>(10, 2, QMC::AccessPublic, QMetaType::Void, {{
            { 0x80000000 | 11, 12 },
        }}),
        // Slot 'updateByteViewSettings'
        QtMocHelpers::SlotData<void()>(13, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'markProtocol'
        QtMocHelpers::SlotData<void(int, int)>(14, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'markField'
        QtMocHelpers::SlotData<void(int, int, bool, bool)>(17, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { QMetaType::Bool, 18 }, { QMetaType::Bool, 19 },
        }}),
        // Slot 'markField'
        QtMocHelpers::SlotData<void(int, int, bool)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 }, { QMetaType::Bool, 18 },
        }}),
        // Slot 'markField'
        QtMocHelpers::SlotData<void(int, int)>(17, 2, QMC::AccessPublic | QMC::MethodCloned, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'markAppendix'
        QtMocHelpers::SlotData<void(int, int)>(20, 2, QMC::AccessPublic, QMetaType::Void, {{
            { QMetaType::Int, 15 }, { QMetaType::Int, 16 },
        }}),
        // Slot 'unmarkField'
        QtMocHelpers::SlotData<void()>(21, 2, QMC::AccessPublic, QMetaType::Void),
        // Slot 'copyBytes'
        QtMocHelpers::SlotData<void(bool)>(22, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 23 },
        }}),
        // Slot 'setHexDisplayFormat'
        QtMocHelpers::SlotData<void(QAction *)>(24, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'setCharacterEncoding'
        QtMocHelpers::SlotData<void(QAction *)>(27, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { 0x80000000 | 25, 26 },
        }}),
        // Slot 'toggleHoverAllowed'
        QtMocHelpers::SlotData<void(bool)>(28, 2, QMC::AccessPrivate, QMetaType::Void, {{
            { QMetaType::Bool, 29 },
        }}),
        // Slot 'requestAddAnnotation'
        QtMocHelpers::SlotData<void()>(30, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestEditAnnotation'
        QtMocHelpers::SlotData<void()>(31, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestRemoveAnnotation'
        QtMocHelpers::SlotData<void()>(32, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestSetOffsetStart'
        QtMocHelpers::SlotData<void()>(33, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestSetOffsetEnd'
        QtMocHelpers::SlotData<void()>(34, 2, QMC::AccessPrivate, QMetaType::Void),
        // Slot 'requestClearOffsetMarkers'
        QtMocHelpers::SlotData<void()>(35, 2, QMC::AccessPrivate, QMetaType::Void),
    };
    QtMocHelpers::UintData qt_properties {
    };
    QtMocHelpers::UintData qt_enums {
    };
    return QtMocHelpers::metaObjectData<HexDataSourceView, qt_meta_tag_ZN17HexDataSourceViewE_t>(QMC::MetaObjectFlag{}, qt_stringData,
            qt_methods, qt_properties, qt_enums);
}
Q_CONSTINIT const QMetaObject HexDataSourceView::staticMetaObject = { {
    QMetaObject::SuperData::link<BaseDataSourceView::staticMetaObject>(),
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17HexDataSourceViewE_t>.stringdata,
    qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17HexDataSourceViewE_t>.data,
    qt_static_metacall,
    nullptr,
    qt_staticMetaObjectRelocatingContent<qt_meta_tag_ZN17HexDataSourceViewE_t>.metaTypes,
    nullptr
} };

void HexDataSourceView::qt_static_metacall(QObject *_o, QMetaObject::Call _c, int _id, void **_a)
{
    auto *_t = static_cast<HexDataSourceView *>(_o);
    if (_c == QMetaObject::InvokeMetaMethod) {
        switch (_id) {
        case 0: _t->byteViewSettingsChanged(); break;
        case 1: _t->addAnnotationRequested(); break;
        case 2: _t->editAnnotationRequested(); break;
        case 3: _t->removeAnnotationRequested(); break;
        case 4: _t->offsetStartRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 5: _t->offsetEndRequested((*reinterpret_cast<std::add_pointer_t<int>>(_a[1]))); break;
        case 6: _t->offsetMarkersCleared(); break;
        case 7: _t->setMonospaceFont((*reinterpret_cast<std::add_pointer_t<QFont>>(_a[1]))); break;
        case 8: _t->updateByteViewSettings(); break;
        case 9: _t->markProtocol((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 10: _t->markField((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[4]))); break;
        case 11: _t->markField((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2])),(*reinterpret_cast<std::add_pointer_t<bool>>(_a[3]))); break;
        case 12: _t->markField((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 13: _t->markAppendix((*reinterpret_cast<std::add_pointer_t<int>>(_a[1])),(*reinterpret_cast<std::add_pointer_t<int>>(_a[2]))); break;
        case 14: _t->unmarkField(); break;
        case 15: _t->copyBytes((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 16: _t->setHexDisplayFormat((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 17: _t->setCharacterEncoding((*reinterpret_cast<std::add_pointer_t<QAction*>>(_a[1]))); break;
        case 18: _t->toggleHoverAllowed((*reinterpret_cast<std::add_pointer_t<bool>>(_a[1]))); break;
        case 19: _t->requestAddAnnotation(); break;
        case 20: _t->requestEditAnnotation(); break;
        case 21: _t->requestRemoveAnnotation(); break;
        case 22: _t->requestSetOffsetStart(); break;
        case 23: _t->requestSetOffsetEnd(); break;
        case 24: _t->requestClearOffsetMarkers(); break;
        default: ;
        }
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        switch (_id) {
        default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
        case 16:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        case 17:
            switch (*reinterpret_cast<int*>(_a[1])) {
            default: *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType(); break;
            case 0:
                *reinterpret_cast<QMetaType *>(_a[0]) = QMetaType::fromType< QAction* >(); break;
            }
            break;
        }
    }
    if (_c == QMetaObject::IndexOfMethod) {
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)()>(_a, &HexDataSourceView::byteViewSettingsChanged, 0))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)()>(_a, &HexDataSourceView::addAnnotationRequested, 1))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)()>(_a, &HexDataSourceView::editAnnotationRequested, 2))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)()>(_a, &HexDataSourceView::removeAnnotationRequested, 3))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)(int )>(_a, &HexDataSourceView::offsetStartRequested, 4))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)(int )>(_a, &HexDataSourceView::offsetEndRequested, 5))
            return;
        if (QtMocHelpers::indexOfMethod<void (HexDataSourceView::*)()>(_a, &HexDataSourceView::offsetMarkersCleared, 6))
            return;
    }
}

const QMetaObject *HexDataSourceView::metaObject() const
{
    return QObject::d_ptr->metaObject ? QObject::d_ptr->dynamicMetaObject() : &staticMetaObject;
}

void *HexDataSourceView::qt_metacast(const char *_clname)
{
    if (!_clname) return nullptr;
    if (!strcmp(_clname, qt_staticMetaObjectStaticContent<qt_meta_tag_ZN17HexDataSourceViewE_t>.strings))
        return static_cast<void*>(this);
    if (!strcmp(_clname, "IDataPrintable"))
        return static_cast< IDataPrintable*>(this);
    if (!strcmp(_clname, "org.wireshark.Qt.UI.IDataPrintable"))
        return static_cast< IDataPrintable*>(this);
    return BaseDataSourceView::qt_metacast(_clname);
}

int HexDataSourceView::qt_metacall(QMetaObject::Call _c, int _id, void **_a)
{
    _id = BaseDataSourceView::qt_metacall(_c, _id, _a);
    if (_id < 0)
        return _id;
    if (_c == QMetaObject::InvokeMetaMethod) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    if (_c == QMetaObject::RegisterMethodArgumentMetaType) {
        if (_id < 25)
            qt_static_metacall(this, _c, _id, _a);
        _id -= 25;
    }
    return _id;
}

// SIGNAL 0
void HexDataSourceView::byteViewSettingsChanged()
{
    QMetaObject::activate(this, &staticMetaObject, 0, nullptr);
}

// SIGNAL 1
void HexDataSourceView::addAnnotationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 1, nullptr);
}

// SIGNAL 2
void HexDataSourceView::editAnnotationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 2, nullptr);
}

// SIGNAL 3
void HexDataSourceView::removeAnnotationRequested()
{
    QMetaObject::activate(this, &staticMetaObject, 3, nullptr);
}

// SIGNAL 4
void HexDataSourceView::offsetStartRequested(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 4, nullptr, _t1);
}

// SIGNAL 5
void HexDataSourceView::offsetEndRequested(int _t1)
{
    QMetaObject::activate<void>(this, &staticMetaObject, 5, nullptr, _t1);
}

// SIGNAL 6
void HexDataSourceView::offsetMarkersCleared()
{
    QMetaObject::activate(this, &staticMetaObject, 6, nullptr);
}
QT_WARNING_POP
